# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import LiveScenarioTest, ScenarioTest, ResourceGroupPreparer, record_only
from knack.util import CLIError


class TestMonitorAutoscaleScenario(ScenarioTest):

    @ResourceGroupPreparer(name_prefix='cli_test_monitor_autoscale')
    def test_monitor_autoscale_basic(self, resource_group):
        self.kwargs.update({
            'vmss': 'vmss1'
        })
        self.cmd('vmss create -g {rg} -n {vmss} --image UbuntuLTS --admin-username testadmin --admin-password TestTest12#$')
        self.kwargs['vmss_id'] = self.cmd('vmss show -g {rg} -n {vmss}').get_output_in_json()['id']

        self.cmd('monitor autoscale create --resource {vmss_id} --count 3', checks=[
            self.check('profiles[0].capacity.default', 3),
            self.check('profiles[0].capacity.minimum', 3),
            self.check('profiles[0].capacity.maximum', 3)
        ])
        self.cmd('monitor autoscale list -g {rg}',
                 checks=self.check('length(@)', 1))
        self.cmd('monitor autoscale show -g {rg} -n {vmss}')

        # verify that count behaves correctly
        self.cmd('monitor autoscale update -g {rg} -n {vmss} --count 2', checks=[
            self.check('profiles[0].capacity.default', 2),
            self.check('profiles[0].capacity.minimum', 2),
            self.check('profiles[0].capacity.maximum', 2)
        ])
        self.cmd('monitor autoscale update -g {rg} -n {vmss} --min-count 1 --count 2 --max-count 4', checks=[
            self.check('profiles[0].capacity.default', 2),
            self.check('profiles[0].capacity.minimum', 1),
            self.check('profiles[0].capacity.maximum', 4)
        ])
        self.cmd('monitor autoscale update -g {rg} -n {vmss} --max-count 5', checks=[
            self.check('profiles[0].capacity.default', 2),
            self.check('profiles[0].capacity.minimum', 1),
            self.check('profiles[0].capacity.maximum', 5)
        ])
        self.cmd('monitor autoscale delete -g {rg} -n {vmss}')

    @ResourceGroupPreparer(name_prefix='cli_test_monitor_autoscale_rules')
    def test_monitor_autoscale_rules(self, resource_group):
        self.kwargs.update({
            'vmss': 'vmss1'
        })
        self.cmd('vmss create -g {rg} -n {vmss} --image UbuntuLTS --admin-username testadmin --admin-password TestTest12#$')
        self.kwargs['vmss_id'] = self.cmd('vmss show -g {rg} -n {vmss}').get_output_in_json()['id']

        self.cmd('monitor autoscale create --resource {vmss_id} --min-count 1 --count 3 --max-count 5')

        self.cmd('monitor autoscale rule list -g {rg} --autoscale-name {vmss}')

        self.cmd('monitor autoscale rule create -g {rg} --autoscale-name {vmss} --condition "Percentage CPU > 75 avg 5m" --scale to 5', checks=[
            self.check('metricTrigger.metricName', 'Percentage CPU'),
            self.check('metricTrigger.operator', 'GreaterThan'),
            self.check('metricTrigger.threshold', 75),
            self.check('metricTrigger.statistic', 'Average'),
            self.check('metricTrigger.timeAggregation', 'Average'),
            self.check('metricTrigger.timeWindow', 'PT5M'),
            self.check('metricTrigger.timeGrain', 'PT1M'),
            self.check('scaleAction.cooldown', 'PT5M'),
            self.check('scaleAction.direction', 'None'),
            self.check('scaleAction.type', 'ExactCount'),
            self.check('scaleAction.value', '5')
        ])
        self.cmd('monitor autoscale rule create -g {rg} --autoscale-name {vmss} --timegrain "avg 5m" --condition "Percentage CPU < 30 avg 10m" --scale in 50% --cooldown 10', checks=[
            self.check('metricTrigger.metricName', 'Percentage CPU'),
            self.check('metricTrigger.operator', 'LessThan'),
            self.check('metricTrigger.threshold', 30),
            self.check('metricTrigger.statistic', 'Average'),
            self.check('metricTrigger.timeAggregation', 'Average'),
            self.check('metricTrigger.timeWindow', 'PT10M'),
            self.check('metricTrigger.timeGrain', 'PT5M'),
            self.check('scaleAction.cooldown', 'PT10M'),
            self.check('scaleAction.direction', 'Decrease'),
            self.check('scaleAction.type', 'PercentChangeCount'),
            self.check('scaleAction.value', '50')
        ])
        self.cmd('monitor autoscale rule create -g {rg} --autoscale-name {vmss} --timegrain "min 1m" --condition "Percentage CPU < 10 avg 5m" --scale to 1', checks=[
            self.check('metricTrigger.metricName', 'Percentage CPU'),
            self.check('metricTrigger.operator', 'LessThan'),
            self.check('metricTrigger.threshold', 10),
            self.check('metricTrigger.statistic', 'Min'),
            self.check('metricTrigger.timeAggregation', 'Average'),
            self.check('metricTrigger.timeWindow', 'PT5M'),
            self.check('metricTrigger.timeGrain', 'PT1M'),
            self.check('scaleAction.cooldown', 'PT5M'),
            self.check('scaleAction.direction', 'None'),
            self.check('scaleAction.type', 'ExactCount'),
            self.check('scaleAction.value', '1')
        ])

        # verify order is stable
        list_1 = self.cmd('monitor autoscale rule list -g {rg} --autoscale-name {vmss}').get_output_in_json()
        with self.assertRaisesRegexp(CLIError, 'Please double check the name of the autoscale profile.'):
            self.cmd('monitor autoscale rule list -g {rg} --autoscale-name {vmss} --profile-name falseprofile')

        list_2 = self.cmd('monitor autoscale rule list -g {rg} --autoscale-name {vmss}').get_output_in_json()
        self.assertTrue(len(list_1) == 3 and len(list_2) == 3)
        for x in range(len(list_1)):
            self.assertTrue(list_1[x] == list_2[x])

        # verify copy works
        self.cmd('monitor autoscale profile create -g {rg} --autoscale-name {vmss} -n test2 --start 2018-03-01 --end 2018-04-01 --min-count 1 --count 3 --max-count 5 --timezone "Pacific Standard Time"')
        self.cmd('monitor autoscale profile create -g {rg} --autoscale-name {vmss} -n test3 --start 2018-05-01 --end 2018-06-01 --min-count 1 --count 2 --max-count 5 --timezone "Pacific Standard Time"')
        self.cmd('monitor autoscale profile create -g {rg} --autoscale-name {vmss} -n test1 --start 2018-01-01 --end 2018-02-01 --min-count 1 --count 2 --max-count 5 --timezone "Pacific Standard Time" --copy-rules default')

        self.cmd('monitor autoscale rule list -g {rg} --autoscale-name {vmss} --profile-name test1',
                 checks=self.check('length(@)', 3))
        self.cmd('monitor autoscale rule list -g {rg} --autoscale-name {vmss} --profile-name test2',
                 checks=self.check('length(@)', 0))
        self.cmd('monitor autoscale rule list -g {rg} --autoscale-name {vmss} --profile-name test3',
                 checks=self.check('length(@)', 0))

        self.cmd('monitor autoscale rule copy -g {rg} --autoscale-name {vmss} --source-schedule test1 --dest-schedule test2 --index "*"')
        self.cmd('monitor autoscale rule copy -g {rg} --autoscale-name {vmss} --source-schedule test2 --dest-schedule test3 --index 0')

        self.cmd('monitor autoscale rule list -g {rg} --autoscale-name {vmss} --profile-name test2',
                 checks=self.check('length(@)', 3))
        self.cmd('monitor autoscale rule list -g {rg} --autoscale-name {vmss} --profile-name test3',
                 checks=self.check('length(@)', 1))

        # verify rule removal by index and remove all works
        self.cmd('monitor autoscale rule delete -g {rg} --autoscale-name {vmss} --index 2')
        list_3 = self.cmd('monitor autoscale rule list -g {rg} --autoscale-name {vmss}').get_output_in_json()
        self.assertTrue(len(list_3) == 2)

        self.cmd('monitor autoscale rule delete -g {rg} --autoscale-name {vmss} --index "*"')
        list_4 = self.cmd('monitor autoscale rule list -g {rg} --autoscale-name {vmss}').get_output_in_json()
        self.assertTrue(len(list_4) == 0)

    @ResourceGroupPreparer(name_prefix='cli_test_monitor_autoscale_rule_with_dimensions')
    def test_monitor_autoscale_rule_with_dimensions(self, resource_group):
        self.kwargs.update({
            'vmss': 'vmss1'
        })
        self.cmd(
            'vmss create -g {rg} -n {vmss} --image UbuntuLTS --admin-username testadmin --admin-password TestTest12#$ --instance-count 2')
        self.kwargs['vmss_id'] = self.cmd('vmss show -g {rg} -n {vmss}').get_output_in_json()['id']

        self.cmd('monitor autoscale create --resource {vmss_id} --min-count 1 --count 3 --max-count 5')

        self.cmd('monitor autoscale rule list -g {rg} --autoscale-name {vmss}')

        self.cmd(
            'monitor autoscale rule create -g {rg} --autoscale-name {vmss} --condition "\'Mynamespace.abcd\' Percentage CPU > 75 avg 5m where VMName == cliname1 or cliname2" --scale to 5',
            checks=[
                self.check('metricTrigger.metricName', 'Percentage CPU'),
                self.check('metricTrigger.operator', 'GreaterThan'),
                self.check('metricTrigger.threshold', 75),
                self.check('metricTrigger.statistic', 'Average'),
                self.check('metricTrigger.timeAggregation', 'Average'),
                self.check('metricTrigger.timeWindow', 'PT5M'),
                self.check('metricTrigger.timeGrain', 'PT1M'),
                self.check('metricTrigger.dimensions[0].dimensionName', 'VMName'),
                self.check('metricTrigger.dimensions[0].operator', 'Equals'),
                self.check('metricTrigger.dimensions[0].values[0]', 'cliname1'),
                self.check('metricTrigger.dimensions[0].values[1]', 'cliname2'),
                self.check('metricTrigger.metricNamespace', 'Mynamespace.abcd'),
                self.check('scaleAction.cooldown', 'PT5M'),
                self.check('scaleAction.direction', 'None'),
                self.check('scaleAction.type', 'ExactCount'),
                self.check('scaleAction.value', '5')
            ])

        self.cmd(
            'monitor autoscale rule create -g {rg} --autoscale-name {vmss} --condition "\'Mynamespace.abcd\' Percentage CPU > 75 avg 5m where VMName == cliname1 or cliname2" --scale to 5',
            checks=[
                self.check('metricTrigger.metricName', 'Percentage CPU'),
                self.check('metricTrigger.operator', 'GreaterThan'),
                self.check('metricTrigger.threshold', 75),
                self.check('metricTrigger.statistic', 'Average'),
                self.check('metricTrigger.timeAggregation', 'Average'),
                self.check('metricTrigger.timeWindow', 'PT5M'),
                self.check('metricTrigger.timeGrain', 'PT1M'),
                self.check('metricTrigger.dimensions[0].dimensionName', 'VMName'),
                self.check('metricTrigger.dimensions[0].operator', 'Equals'),
                self.check('metricTrigger.dimensions[0].values[0]', 'cliname1'),
                self.check('metricTrigger.dimensions[0].values[1]', 'cliname2'),
                self.check('metricTrigger.metricNamespace', 'Mynamespace.abcd'),
                self.check('scaleAction.cooldown', 'PT5M'),
                self.check('scaleAction.direction', 'None'),
                self.check('scaleAction.type', 'ExactCount'),
                self.check('scaleAction.value', '5')
            ])

        self.cmd(
            'monitor autoscale rule create -g {rg} --autoscale-name {vmss} --condition "\'Mynamespace.abcd\' Percentage CPU > 75 avg 5m where VMName == cliname1 or cliname2" --scale to 5',
            checks=[
                self.check('metricTrigger.metricName', 'Percentage CPU'),
                self.check('metricTrigger.operator', 'GreaterThan'),
                self.check('metricTrigger.threshold', 75),
                self.check('metricTrigger.statistic', 'Average'),
                self.check('metricTrigger.timeAggregation', 'Average'),
                self.check('metricTrigger.timeWindow', 'PT5M'),
                self.check('metricTrigger.timeGrain', 'PT1M'),
                self.check('metricTrigger.dimensions[0].dimensionName', 'VMName'),
                self.check('metricTrigger.dimensions[0].operator', 'Equals'),
                self.check('metricTrigger.dimensions[0].values[0]', 'cliname1'),
                self.check('metricTrigger.dimensions[0].values[1]', 'cliname2'),
                self.check('metricTrigger.metricNamespace', 'Mynamespace.abcd'),
                self.check('scaleAction.cooldown', 'PT5M'),
                self.check('scaleAction.direction', 'None'),
                self.check('scaleAction.type', 'ExactCount'),
                self.check('scaleAction.value', '5')
            ])

        self.cmd('monitor autoscale rule list -g {rg} --autoscale-name {vmss}', checks=[
            self.check('length(@)', 3)
        ])

    @ResourceGroupPreparer(name_prefix='cli_test_monitor_autoscale_fixed')
    def test_monitor_autoscale_fixed(self, resource_group):
        self.kwargs.update({
            'vmss': 'vmss1',
            'sched': 'Christmas'
        })
        self.cmd('vmss create -g {rg} -n {vmss} --image UbuntuLTS --admin-username testadmin --admin-password TestTest12#$')
        self.kwargs['vmss_id'] = self.cmd('vmss show -g {rg} -n {vmss}').get_output_in_json()['id']

        self.cmd('monitor autoscale create --resource {vmss_id} --count 3')

        self.cmd('monitor autoscale profile create -g {rg} --autoscale-name {vmss} -n {sched} --start 2018-12-24 --end 2018-12-26 --count 5 --timezone "Pacific Standard Time"', checks=[
            self.check('capacity.default', 5),
            self.check('capacity.minimum', 5),
            self.check('capacity.maximum', 5),
            self.check('fixedDate.end', '2018-12-26T00:00:00+00:00'),
            self.check('fixedDate.start', '2018-12-24T00:00:00+00:00'),
            self.check('fixedDate.timeZone', 'Pacific Standard Time'),
            self.check('recurrence', None)
        ])

        self.cmd('monitor autoscale profile list -g {rg} --autoscale-name {vmss}',
                 checks=self.check('length(@)', 2))
        self.cmd('monitor autoscale profile delete -g {rg} --autoscale-name {vmss} -n {sched}')
        self.cmd('monitor autoscale profile list -g {rg} --autoscale-name {vmss}',
                 checks=self.check('length(@)', 1))

    @ResourceGroupPreparer(name_prefix='cli_test_monitor_autoscale_recurring')
    def test_monitor_autoscale_recurring(self, resource_group):
        import json
        import time

        sleep_time = 3

        self.kwargs.update({
            'vmss': 'vmss1'
        })
        self.cmd('vmss create -g {rg} -n {vmss} --image UbuntuLTS --admin-username testname --admin-password TestTest12#$')
        self.kwargs['vmss_id'] = self.cmd('vmss show -g {rg} -n {vmss}').get_output_in_json()['id']

        self.cmd('monitor autoscale create --resource {vmss_id} --count 3')
        time.sleep(sleep_time)

        self.cmd('monitor autoscale profile create -g {rg} --autoscale-name {vmss} -n weekend --recurrence week sat sun --count 1 --timezone "Pacific Standard Time"')
        time.sleep(sleep_time)

        self.cmd('monitor autoscale profile create -g {rg} --autoscale-name {vmss} -n weekday --recurrence week mo tu we th fr --count 4 --timezone "Pacific Standard Time"')
        time.sleep(sleep_time)

        # 2 profiles + 2 "default" profiles + default "default" profile
        self.cmd('monitor autoscale profile list -g {rg} --autoscale-name {vmss}',
                 checks=self.check('length(@)', 5))

        # should update all "default" profiles
        value = 4
        self.cmd('monitor autoscale update -g {{rg}} -n {{vmss}} --count {}'.format(value))
        time.sleep(sleep_time)

        schedules = self.cmd('monitor autoscale profile list -g {rg} --autoscale-name {vmss}').get_output_in_json()

        def _is_default(val):
            if not val['fixedDate'] and not val['recurrence']:
                return True
            try:
                json.loads(val['name'])
                return True
            except ValueError:
                return False

        for schedule in [x for x in schedules if _is_default(x)]:
            self.assertTrue(int(schedule['capacity']['default']) == value)
            self.assertTrue(int(schedule['capacity']['minimum']) == value)
            self.assertTrue(int(schedule['capacity']['maximum']) == value)

        # should delete the weekend profile and its matching default
        self.cmd('monitor autoscale profile delete -g {rg} --autoscale-name {vmss} -n weekend')
        time.sleep(sleep_time)
        self.cmd('monitor autoscale profile list -g {rg} --autoscale-name {vmss}',
                 checks=self.check('length(@)', 3))

        # should delete the weekday profile and its matching default
        self.cmd('monitor autoscale profile delete -g {rg} --autoscale-name {vmss} -n weekday')
        time.sleep(sleep_time)
        self.cmd('monitor autoscale profile list -g {rg} --autoscale-name {vmss}',
                 checks=self.check('length(@)', 1))


# inexplicably fails on CI so making into a live test
class TestMonitorAutoscaleTimezones(LiveScenarioTest):

    def test_monitor_autoscale_timezones(self):
        self.cmd('monitor autoscale profile list-timezones',
                 checks=self.check('length(@)', 136))
        self.cmd('monitor autoscale profile list-timezones -q pacific',
                 checks=self.check('length(@)', 6))
        self.cmd('monitor autoscale profile list-timezones --offset +12',
                 checks=self.check('length(@)', 6))
        self.cmd('monitor autoscale profile list-timezones -q pacific --offset -4',
                 checks=self.check('length(@)', 1))


class TestMonitorAutoscaleComplexRules(ScenarioTest):
    def setUp(self):
        super(TestMonitorAutoscaleComplexRules, self).setUp()
        self.cmd('extension add -n spring-cloud')

    def tearDown(self):
        self.cmd('extension remove -n spring-cloud')
        super(TestMonitorAutoscaleComplexRules, self).tearDown()

#    @record_only()
    @ResourceGroupPreparer(name_prefix='cli_test_monitor_autoscale_rule_for_spring_cloud', location='westus2')
    def test_monitor_autoscale_rule_for_spring_cloud(self, resource_group):
        self.kwargs.update({
            'sc': self.create_random_name('clitestsc', 15),
            'rg': resource_group,
            'scapp': 'app1',
            'gitrepo': 'https://github.com/Azure-Samples/piggymetrics-config',
        })

        self.cmd('spring-cloud create -n {sc} -g {rg}')
        self.cmd('spring-cloud config-server git set -n {sc} -g {rg} --uri {gitrepo}')
        self.kwargs['deployment_id'] = self.cmd('spring-cloud app create -n {scapp} -s {sc} -g {rg}').get_output_in_json()['properties']['activeDeployment']['id']

        self.cmd('monitor autoscale create -g {rg} --resource {deployment_id} --min-count 1 --count 1 --max-count 3')

        self.cmd('monitor autoscale rule list -g {rg} --autoscale-name {sc}')

        self.cmd(
            'monitor autoscale rule create -g {rg} --autoscale-name {sc} --condition "tomcat.global.request.total.count > 0 avg 3m where AppName == {scapp} and Deployment == default" --scale out 1',
            checks=[
                self.check('metricTrigger.metricName', 'tomcat.global.request.total.count'),
                self.check('metricTrigger.metricNamespace', 'Microsoft.AppPlatform/Spring'),
                self.check('metricTrigger.operator', 'GreaterThan'),
                self.check('metricTrigger.threshold', 0),
                self.check('metricTrigger.statistic', 'Average'),
                self.check('metricTrigger.timeAggregation', 'Average'),
                self.check('metricTrigger.timeWindow', 'PT3M'),
                self.check('metricTrigger.timeGrain', 'PT1M'),
                self.check('metricTrigger.dimensions[0].dimensionName', 'AppName'),
                self.check('metricTrigger.dimensions[0].operator', 'Equals'),
                self.check('metricTrigger.dimensions[0].values[0]', self.kwargs['scapp']),
                self.check('metricTrigger.dimensions[1].dimensionName', 'Deployment'),
                self.check('metricTrigger.dimensions[1].operator', 'Equals'),
                self.check('metricTrigger.dimensions[1].values[0]', 'default'),
                self.check('scaleAction.cooldown', 'PT5M'),
                self.check('scaleAction.direction', 'Increase'),
                self.check('scaleAction.type', 'ChangeCount'),
                self.check('scaleAction.value', '1')
            ])

        self.cmd(
            'monitor autoscale rule create -g {rg} --autoscale-name {sc} --condition "tomcat.global.request.total.count > 0 avg 3m where AppName == {scapp} and Deployment == default" --scale out 1',
            checks=[
                self.check('metricTrigger.metricName', 'tomcat.global.request.total.count'),
                self.check('metricTrigger.metricNamespace', 'Microsoft.AppPlatform/Spring'),
                self.check('metricTrigger.operator', 'GreaterThan'),
                self.check('metricTrigger.threshold', 0),
                self.check('metricTrigger.statistic', 'Average'),
                self.check('metricTrigger.timeAggregation', 'Average'),
                self.check('metricTrigger.timeWindow', 'PT3M'),
                self.check('metricTrigger.timeGrain', 'PT1M'),
                self.check('metricTrigger.dimensions[0].dimensionName', 'AppName'),
                self.check('metricTrigger.dimensions[0].operator', 'Equals'),
                self.check('metricTrigger.dimensions[0].values[0]', self.kwargs['scapp']),
                self.check('metricTrigger.dimensions[1].dimensionName', 'Deployment'),
                self.check('metricTrigger.dimensions[1].operator', 'Equals'),
                self.check('metricTrigger.dimensions[1].values[0]', 'default'),
                self.check('scaleAction.cooldown', 'PT5M'),
                self.check('scaleAction.direction', 'Increase'),
                self.check('scaleAction.type', 'ChangeCount'),
                self.check('scaleAction.value', '1')
            ])

        self.cmd(
            'monitor autoscale rule create -g {rg} --autoscale-name {sc} --condition "tomcat.global.request.total.count > 0 avg 3m where AppName == {scapp} and Deployment == default" --scale out 1',
            checks=[
                self.check('metricTrigger.metricName', 'tomcat.global.request.total.count'),
                self.check('metricTrigger.metricNamespace', 'Microsoft.AppPlatform/Spring'),
                self.check('metricTrigger.operator', 'GreaterThan'),
                self.check('metricTrigger.threshold', 0),
                self.check('metricTrigger.statistic', 'Average'),
                self.check('metricTrigger.timeAggregation', 'Average'),
                self.check('metricTrigger.timeWindow', 'PT3M'),
                self.check('metricTrigger.timeGrain', 'PT1M'),
                self.check('metricTrigger.dimensions[0].dimensionName', 'AppName'),
                self.check('metricTrigger.dimensions[0].operator', 'Equals'),
                self.check('metricTrigger.dimensions[0].values[0]', self.kwargs['scapp']),
                self.check('metricTrigger.dimensions[1].dimensionName', 'Deployment'),
                self.check('metricTrigger.dimensions[1].operator', 'Equals'),
                self.check('metricTrigger.dimensions[1].values[0]', 'default'),
                self.check('scaleAction.cooldown', 'PT5M'),
                self.check('scaleAction.direction', 'Increase'),
                self.check('scaleAction.type', 'ChangeCount'),
                self.check('scaleAction.value', '1')
            ])

        self.cmd('monitor autoscale rule list -g {rg} --autoscale-name {sc}', checks=[
            self.check('length(@)', 3)
        ])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from setuptools import setup

VERSION = "0.1.1"

CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'Intended Audience :: System Administrators',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'License :: OSI Approved :: MIT License',
]

# Until https://gitlab.com/pycqa/flake8/issues/415 is resolved, pin version of pycodestyle
DEPENDENCIES = [
    'coverage>=4.2',
    'flake8==3.5.0',
    'pycodestyle==2.3.1',
    'nose>=1.3.7',
    'readme_renderer>=17.2',
    'requests',
    'pyyaml~=5.2',
    'knack',
    'six>=1.10.0',
    'tabulate>=0.7.7',
    'colorama>=0.3.7'
]

setup(
    name='azure-cli-dev-tools',
    version=VERSION,
    description='Microsoft Azure Command-Line - Development Tools',
    long_description='',
    license='MIT',
    author='Microsoft Corporation',
    author_email='azpycli@microsoft.com',
    url='https://github.com/Azure/azure-cli',
    packages=[
        'automation',
        'automation.style',
        'automation.tests',
        'automation.setup',
        'automation.coverage',
        'automation.verify'
    ],
    entry_points={
        'console_scripts': [
            'azdev=automation.__main__:main',
            'check_style=automation.style:legacy_entry',
            'run_tests=automation.tests:legacy_entry_point'
        ]
    },
    install_requires=DEPENDENCIES,
    extras_require={ 
        ":python_version<'3.0'": ['pylint==1.9.2'],
        ":python_version>='3.0'": ['pylint==2.0.0']
    }
)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from enum import Enum

import azure.cli.core.telemetry as telemetry
from knack.log import get_logger


logger = get_logger(__name__)


class AladdinUserFaultType(Enum):
    """Define the userfault types required by aladdin service
    to get the command recommendations"""

    ExpectedArgument = 'ExpectedArgument'
    UnrecognizedArguments = 'UnrecognizedArguments'
    ValidationError = 'ValidationError'
    UnknownSubcommand = 'UnknownSubcommand'
    MissingRequiredParameters = 'MissingRequiredParameters'
    MissingRequiredSubcommand = 'MissingRequiredSubcommand'
    StorageAccountNotFound = 'StorageAccountNotFound'
    Unknown = 'Unknown'
    InvalidJMESPathQuery = 'InvalidJMESPathQuery'
    InvalidOutputType = 'InvalidOutputType'
    InvalidParameterValue = 'InvalidParameterValue'
    UnableToParseCommandInput = 'UnableToParseCommandInput'
    ResourceGroupNotFound = 'ResourceGroupNotFound'
    InvalidDateTimeArgumentValue = 'InvalidDateTimeArgumentValue'
    InvalidResourceGroupName = 'InvalidResourceGroupName'
    AzureResourceNotFound = 'AzureResourceNotFound'
    InvalidAccountName = 'InvalidAccountName'


class CommandRecommender():
    """Recommend a command for user when user's command fails.
    It combines Aladdin recommendations and examples in help files."""

    def __init__(self, command, parameters, extension, error_msg, cli_ctx):
        """
        :param command: The command name in user's input.
        :type command: str
        :param parameters: The raw parameters in users input.
        :type parameters: list
        :param extension: The extension name in user's input if the command comes from an extension.
        :type extension: str
        :param error_msg: The error message of the failed command.
        :type error_msg: str
        :param cli_ctx: CLI context when parser fails.
        :type cli_ctx: knack.cli.CLI
        """
        self.command = command.strip()
        self.parameters = parameters
        self.extension = extension
        self.error_msg = error_msg
        self.cli_ctx = cli_ctx

        self.help_examples = []
        self.aladdin_recommendations = []

    def set_help_examples(self, examples):
        """Set recommendations from help files"""

        self.help_examples.extend(examples)

    def _set_aladdin_recommendations(self):
        """Set recommendations from aladdin service.
        Call the aladdin service API, parse the response and set the recommendations.
        """

        import hashlib
        import json
        import requests
        from requests import RequestException
        from http import HTTPStatus
        from azure.cli.core import __version__ as version

        api_url = 'https://app.aladdin.microsoft.com/api/v1.0/suggestions'
        correlation_id = telemetry._session.correlation_id  # pylint: disable=protected-access
        subscription_id = telemetry._get_azure_subscription_id()  # pylint: disable=protected-access
        # Used for DDOS protection and rate limiting
        user_id = telemetry._get_user_azure_id()  # pylint: disable=protected-access
        hashed_user_id = hashlib.sha256(user_id.encode('utf-8')).hexdigest()

        headers = {
            'Content-Type': 'application/json',
            'X-UserId': hashed_user_id
        }
        context = {
            'versionNumber': version,
            'errorType': self._get_error_type()
        }

        if telemetry.is_telemetry_enabled():
            if correlation_id:
                context['correlationId'] = correlation_id
            if subscription_id:
                context['subscriptionId'] = subscription_id

        parameters = self._normalize_parameters(self.parameters)
        parameters = [item for item in parameters if item not in ['--debug', '--verbose', '--only-show-errors']]
        query = {
            "command": self.command,
            "parameters": ','.join(parameters)
        }

        response = None
        try:
            response = requests.get(
                api_url,
                params={
                    'query': json.dumps(query),
                    'clientType': 'AzureCli',
                    'context': json.dumps(context)
                },
                headers=headers,
                timeout=1)
            telemetry.set_debug_info('AladdinResponseTime', response.elapsed.total_seconds())

        except RequestException as ex:
            logger.debug('Recommendation requests.get() exception: %s', ex)
            telemetry.set_debug_info('AladdinException', ex.__class__.__name__)

        recommendations = []
        if response and response.status_code == HTTPStatus.OK:
            for result in response.json():
                # parse the response and format the recommendation
                command, parameters, placeholders = result['command'],\
                    result['parameters'].split(','),\
                    result['placeholders'].split('♠')
                recommendation = 'az {} '.format(command)
                for parameter, placeholder in zip(parameters, placeholders):
                    recommendation += '{} {} '.format(parameter, placeholder)
                recommendations.append(recommendation.strip())

        self.aladdin_recommendations.extend(recommendations)

    def recommend_a_command(self):
        """Recommend a command for user when user's command fails.
        The recommended command will be the best matched one from
        both the help files and the aladdin recommendations.
        """

        def get_sorted_candidates(target_args, candidate_args_list):
            """Get the sorted candidates by target arguments"""

            candidates = []
            for index, candidate_args in enumerate(candidate_args_list):
                matches = 0
                for arg in candidate_args:
                    if arg in target_args:
                        matches += 1
                candidates.append({
                    'candidate_args': candidate_args,
                    'index': index,
                    'matches': matches
                })

            # sort the candidates by the number of matched arguments and total arguments
            candidates.sort(key=lambda item: (item['matches'], -len(item['candidate_args'])), reverse=True)

            return candidates

        # get recommendations from Aladdin service
        if not self._disable_aladdin_service():
            self._set_aladdin_recommendations()

        recommend_command = ''
        if self.help_examples and self.aladdin_recommendations:
            # all the recommended commands from help examples and aladdin
            all_commands = self.help_examples + self.aladdin_recommendations

            candidate_commands = []
            candidate_args_list = []
            target_args = self._normalize_parameters(self.parameters)
            example_command_name = self.help_examples[0].split(' -')[0]

            for command in all_commands:
                # keep only the commands which begin with a same command name with examples
                if command.startswith(example_command_name):
                    candidate_args_list.append(self._normalize_parameters(command.split(' ')))
                    candidate_commands.append(command)

            # sort the candidates by the number of matched arguments and total arguments
            candidates = get_sorted_candidates(target_args, candidate_args_list)
            if candidates:
                index = candidates[0]['index']
                recommend_command = candidate_commands[index]

        # fallback to use the first recommended command from Aladdin
        elif self.aladdin_recommendations:
            recommend_command = self.aladdin_recommendations[0]
        # set the recommend command into Telemetry
        self._set_recommended_command_to_telemetry(recommend_command)
        # replace the parameter values
        recommend_command = self._replace_parameter_values(recommend_command)

        return recommend_command

    def _set_recommended_command_to_telemetry(self, recommend_command):
        """Set the recommended command to Telemetry for analysis. """

        if recommend_command in self.aladdin_recommendations:
            telemetry.set_debug_info('AladdinRecommendCommand', recommend_command)
        elif recommend_command:
            telemetry.set_debug_info('ExampleRecommendCommand', recommend_command)

    def _disable_aladdin_service(self):
        """Decide whether to disable aladdin request when a command fails.
        The possible cases to disable it are:
            1. In air-gapped clouds
            2. In testing environments
        """
        from azure.cli.core.cloud import CLOUDS_FORBIDDING_ALADDIN_REQUEST

        # CLI is not started well
        if not self.cli_ctx or not self.cli_ctx.cloud:
            return True

        # for air-gapped clouds
        if self.cli_ctx.cloud.name in CLOUDS_FORBIDDING_ALADDIN_REQUEST:
            return True

        # for testing environments
        if self.cli_ctx.__class__.__name__ == 'DummyCli':
            return True

        return False

    def _get_parameter_mappings(self):
        """Get the short option to long option mappings of a command. """

        from knack.deprecation import Deprecated

        try:
            cmd_table = self.cli_ctx.invocation.commands_loader.command_table.get(self.command, None)
            parameter_table = cmd_table.arguments if cmd_table else None
        except AttributeError:
            parameter_table = None

        param_mappings = {
            '-h': '--help',
            '-o': '--output',
            '--only-show-errors': None,
            '--help': None,
            '--output': None,
            '--query': None,
            '--debug': None,
            '--verbose': None
        }

        if parameter_table:
            for argument in parameter_table.values():
                options = argument.type.settings['options_list']
                options = [option for option in options if not isinstance(option, Deprecated)]
                # skip the positional arguments
                if not options:
                    continue
                try:
                    sorted_options = sorted(options, key=len, reverse=True)
                    standard_form = sorted_options[0]

                    for option in sorted_options[1:]:
                        param_mappings[option] = standard_form
                    param_mappings[standard_form] = standard_form
                except TypeError:
                    logger.debug('Unexpected argument options `%s` of type `%s`.', options, type(options).__name__)

        return param_mappings

    def _normalize_parameters(self, raw_parameters):
        """Normalize a parameter list.
        Get the standard parameter names of the raw parameters, which includes:
            1. Use long options to replace short options
            2. Remove the unrecognized parameter names
        An example: ['-g', 'RG', '-n', 'NAME'] ==> {'--resource-group': 'RG', '--name': 'NAME'}
        """

        parameters = self._extract_parameter_names(raw_parameters)
        normalized_parameters = []

        param_mappings = self._get_parameter_mappings()
        for parameter in parameters:
            if parameter in param_mappings:
                normalized_form = param_mappings.get(parameter, None) or parameter
                normalized_parameters.append(normalized_form)
            else:
                logger.debug('"%s" is an invalid parameter for command "%s".', parameter, self.command)

        return sorted(normalized_parameters)

    def _get_error_type(self):
        """The the error type of the failed command from the error message.
        The error types are only consumed by aladdin service for better recommendations.
        """

        error_type = AladdinUserFaultType.Unknown
        if not self.error_msg:
            return error_type.value

        error_msg = self.error_msg.lower()
        if 'unrecognized' in error_msg:
            error_type = AladdinUserFaultType.UnrecognizedArguments
        elif 'expected one argument' in error_msg or 'expected at least one argument' in error_msg \
                or 'value required' in error_msg:
            error_type = AladdinUserFaultType.ExpectedArgument
        elif 'misspelled' in error_msg:
            error_type = AladdinUserFaultType.UnknownSubcommand
        elif 'arguments are required' in error_msg or 'argument required' in error_msg:
            error_type = AladdinUserFaultType.MissingRequiredParameters
            if '_subcommand' in error_msg:
                error_type = AladdinUserFaultType.MissingRequiredSubcommand
            elif '_command_package' in error_msg:
                error_type = AladdinUserFaultType.UnableToParseCommandInput
        elif 'not found' in error_msg or 'could not be found' in error_msg \
                or 'resource not found' in error_msg:
            error_type = AladdinUserFaultType.AzureResourceNotFound
            if 'storage_account' in error_msg or 'storage account' in error_msg:
                error_type = AladdinUserFaultType.StorageAccountNotFound
            elif 'resource_group' in error_msg or 'resource group' in error_msg:
                error_type = AladdinUserFaultType.ResourceGroupNotFound
        elif 'pattern' in error_msg or 'is not a valid value' in error_msg or 'invalid' in error_msg:
            error_type = AladdinUserFaultType.InvalidParameterValue
            if 'jmespath_type' in error_msg:
                error_type = AladdinUserFaultType.InvalidJMESPathQuery
            elif 'datetime_type' in error_msg:
                error_type = AladdinUserFaultType.InvalidDateTimeArgumentValue
            elif '--output' in error_msg:
                error_type = AladdinUserFaultType.InvalidOutputType
            elif 'resource_group' in error_msg:
                error_type = AladdinUserFaultType.InvalidResourceGroupName
            elif 'storage_account' in error_msg:
                error_type = AladdinUserFaultType.InvalidAccountName
        elif "validation error" in error_msg:
            error_type = AladdinUserFaultType.ValidationError

        return error_type.value

    def _extract_parameter_names(self, parameters):  # pylint: disable=no-self-use
        """Extract parameter names from the raw parameters.
        An example: ['-g', 'RG', '-n', 'NAME'] ==> ['-g', '-n']
        """

        from azure.cli.core.commands import AzCliCommandInvoker

        return AzCliCommandInvoker._extract_parameter_names(parameters)  # pylint: disable=protected-access

    def _replace_parameter_values(self, command):
        """Replace the parameter values in recommended command with values in user's command
        An example:
            recommended command: 'az vm create -n MyVm -g MyResourceGroup --image CentOS'
            user's command:  'az vm create --name user_vm -g user_rg'
            ==> 'az vm create -n user_vm -g user_rg --image CentOS'
        """

        def get_parameter_kwargs(parameters):
            """Get name value mappings from parameter list
            An example:
                ['-g', 'RG', '--name=NAME'] ==> {'-g': 'RG', '--name': 'NAME'}
            """

            parameter_kwargs = dict()
            for index, parameter in enumerate(parameters):
                if parameter.startswith('-'):

                    param_name, param_val = parameter, None
                    if '=' in parameter:
                        pieces = parameter.split('=')
                        param_name, param_val = pieces[0], pieces[1]
                    elif index + 1 < len(parameters) and not parameters[index + 1].startswith('-'):
                        param_val = parameters[index + 1]

                    parameter_kwargs[param_name] = param_val

            return parameter_kwargs

        def get_user_param_value(target_param, user_kwargs, param_mappings):
            """Get user's input value for the target_param. """

            standard_user_kwargs = dict()

            for param, val in user_kwargs.items():
                if param in param_mappings:
                    standard_param = param_mappings[param]
                    standard_user_kwargs[standard_param] = val

            if target_param in param_mappings:
                standard_target_param = param_mappings[target_param]
                if standard_target_param in standard_user_kwargs:
                    return standard_user_kwargs[standard_target_param]

            return None

        user_kwargs = get_parameter_kwargs(self.parameters)
        param_mappings = self._get_parameter_mappings()

        command_args = command.split(' ')
        for index, arg in enumerate(command_args):
            if arg.startswith('-') and index + 1 < len(command_args) and not command_args[index + 1].startswith('-'):

                user_param_val = get_user_param_value(arg, user_kwargs, param_mappings)
                if user_param_val:
                    command_args[index + 1] = user_param_val
                else:
                    command_args[index + 1] = '<{}>'.format(command_args[index + 1])

        return ' '.join(command_args)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from azure.cli.testsdk import ScenarioTest


class CdnEdgeNodecenarioTest(ScenarioTest):
    def test_edge_node_crud(self):
        self.cmd('cdn edge-node list', checks=self.check('length(@)', 3))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import unittest
import json

from azure.cli.core.commands.query_examples import QueryTreeBuilder
from azure.cli.core.commands.query_examples import QueryExample


class MockConfig:
    def __init__(self):
        self._config = {
            'examples_len': '80',
            'help_len': '80',
            'max_examples': '-1'
        }

    def get(self, module, field, default_value):
        if module != 'query':
            return default_value
        return self._config.get(field, default_value)

    def set(self, field, value):
        if field in self._config:
            self._config[field] = value


class TestQueryExamplesQueryTreeBuilder(unittest.TestCase):
    def examples_to_dict(self, examples):
        ans = {}
        for ex in examples:
            ans[ex['query string']] = ex
        return ans

    def test_do_parse_dict(self):
        input_json = '''
        {
            "name": {
                "firstName": "Ann",
                "lastName": "Bnn"
            }
        }
        '''
        json_obj = json.loads(input_json)
        mock_config = MockConfig()
        builder = QueryTreeBuilder(mock_config)
        builder.build(json_obj)
        self.assertIsNotNone(builder._root)
        self.assertIsNotNone(builder._all_nodes)
        self.assertIsInstance(builder._all_nodes, dict)
        self.assertIn('name', builder._all_nodes)
        self.assertIn('firstName', builder._all_nodes)
        self.assertIn('lastName', builder._all_nodes)

    def test_do_parse_list(self):
        input_json = '''
        [{
            "name": {
                "firstName": "Ann",
                "lastName": "Bnn"
            }
        }]
        '''
        json_obj = json.loads(input_json)
        mock_config = MockConfig()
        builder = QueryTreeBuilder(mock_config)
        builder.build(json_obj)
        self.assertIsNotNone(builder._root)
        self.assertIsNotNone(builder._all_nodes)
        self.assertIsInstance(builder._all_nodes, dict)
        self.assertIn('name', builder._all_nodes)
        self.assertIn('firstName', builder._all_nodes)
        self.assertIn('lastName', builder._all_nodes)
        # check list
        self.assertTrue(builder._root._is_array)

    def test_parse_empty(self):
        mock_config = MockConfig()
        empty_dict = '{}'
        json_obj = json.loads(empty_dict)
        builder = QueryTreeBuilder(mock_config)
        builder.build(json_obj)
        self.assertIsNotNone(builder._root)  # an empty node
        self.assertFalse(builder._root._child)

        empty_list = '[]'
        json_obj = json.loads(empty_list)
        builder = QueryTreeBuilder(mock_config)
        builder.build(json_obj)
        self.assertIsNone(builder._root)  # No node found

        empty_list = '[{},{}]'
        json_obj = json.loads(empty_list)
        builder = QueryTreeBuilder(mock_config)
        builder.build(json_obj)
        self.assertIsNotNone(builder._root)  # an empty node
        self.assertFalse(builder._root._child)

    def test_dict_recommend(self):
        mock_config = MockConfig()
        keyword_list = []
        output_format = 'json'

        input_json = '''
        {
            "name": {
                "firstName": "Ann"
            },
            "age": 20
        }
        '''
        json_obj = json.loads(input_json)
        builder = QueryTreeBuilder(mock_config)
        builder.build(json_obj)
        examples_list = builder.generate_examples(keyword_list, output_format)
        examples = self.examples_to_dict(examples_list)
        self.assertIn('name', examples)
        self.assertIn('name.firstName', examples)
        self.assertIn('age', examples)

        input_json = '''
        {
            "name": {
                "firstName": "Ann",
                "lastName": "King"
            },
            "age": 20,
            "location":"us",
            "state": "ABC",
            "major": "CS"
        }
        '''
        json_obj = json.loads(input_json)
        max_examples = 3
        mock_config.set('max_examples', max_examples)
        builder = QueryTreeBuilder(mock_config)
        builder.build(json_obj)
        examples_list = builder.generate_examples(keyword_list, output_format)
        self.assertEqual(len(examples_list), max_examples)
        mock_config.set('max_examples', -1)

    def test_array_recommend(self):
        mock_config = MockConfig()
        keyword_list = []
        output_format = 'json'

        input_json = '''
        [{
            "name": {
                "firstName": "Ann"
            },
            "age": "20",
            "friends": [
                {
                    "Name": "Bob"
                }
            ]
        }]
        '''
        json_obj = json.loads(input_json)
        builder = QueryTreeBuilder(mock_config)
        builder.build(json_obj)
        examples_list = builder.generate_examples(keyword_list, output_format)
        examples = self.examples_to_dict(examples_list)
        self.assertIn('[].name', examples)
        self.assertIn('[].name.firstName', examples)
        self.assertIn("[?name.firstName=='Ann']", examples)
        self.assertIn('[].age', examples)
        self.assertIn("[?age=='20']", examples)
        self.assertIn('[].friends[].Name', examples)
        self.assertIn("[].friends[?Name=='Bob']", examples)

        input_json = '''
        {
            "name": {
                "firstName": "Ann"
            },
            "age": "20",
            "friends": [
                {
                    "Name": "Bob"
                }
            ]
        }
        '''
        json_obj = json.loads(input_json)
        builder = QueryTreeBuilder(mock_config)
        builder.build(json_obj)
        examples_list = builder.generate_examples(keyword_list, output_format)
        examples = self.examples_to_dict(examples_list)
        self.assertIn('name', examples)
        self.assertIn('name.firstName', examples)
        self.assertIn('age', examples)
        self.assertIn('friends[]', examples)
        self.assertIn('friends[].Name', examples)
        self.assertIn("friends[?Name=='Bob']", examples)
        self.assertIn("friends[?contains(@.Name, 'something')==\\`true\\`].Name", examples)

    def test_dict_recommend_with_keywords(self):
        mock_config = MockConfig()
        keyword_list = ['name']
        output_format = 'json'

        input_json = '''
        {
            "name": {
                "firstName": "Ann"
            },
            "age": 20
        }
        '''
        json_obj = json.loads(input_json)
        builder = QueryTreeBuilder(mock_config)
        builder.build(json_obj)
        examples_list = builder.generate_examples(keyword_list, output_format)
        examples = self.examples_to_dict(examples_list)
        self.assertIn('name', examples)
        self.assertIn('name.firstName', examples)
        self.assertNotIn('age', examples)
        keyword_list = ['age']
        examples_list = builder.generate_examples(keyword_list, output_format)
        examples = self.examples_to_dict(examples_list)
        self.assertNotIn('name', examples)
        self.assertNotIn('name.firstName', examples)
        self.assertIn('age', examples)

    def test_query_examples(self):
        query_str = "[?contains(name,'Ann')==`true`]"
        help_str = 'This is the help message.'
        example = QueryExample(query_str, help_str)
        result_query_str = example._asdict().get("query string")
        self.assertEqual(result_query_str, "[?contains(name,'Ann')==\\`true\\`]")

        query_str = "This.is.a.very.very.very.long.query.string"
        help_str = "This is a very very very long help message."
        max_length = 10
        example = QueryExample(query_str, help_str)
        example.set_max_length(max_length, max_length)
        result_query_str = example._asdict().get("query string")
        result_help_str = example._asdict().get("help")
        self.assertLessEqual(len(result_query_str), max_length + 5)  # extra length for dots
        self.assertLessEqual(len(result_help_str), max_length + 5)  # extra length for dots


if __name__ == "__main__":
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest

import pytest


def api_version_constraint(resource_type, **kwargs):
    from .reverse_dependency import get_dummy_cli, get_support_api_version_func

    return unittest.skipUnless(get_support_api_version_func()(get_dummy_cli(), resource_type, **kwargs),
                               "Test not supported by current profile.")


def serial_test():
    """
    Mark the test as serial
    """
    return pytest.mark.serial()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest

try:
    import unittest.mock as mock
except ImportError:
    import mock

from knack.util import CLIError

from azure.cli.command_modules.monitor.operations.autoscale_settings import scaffold_autoscale_settings_parameters


class FilterBuilderTests(unittest.TestCase):
    def test_scaffold_autoscale_settings_parameters(self):
        template = scaffold_autoscale_settings_parameters(None)
        self.assertTrue(template)
        self.assertTrue(isinstance(template, dict))


def _mock_get_subscription_id(_):
    return '00000000-0000-0000-0000-000000000000'


class MonitorNameOrIdTest(unittest.TestCase):
    def _build_namespace(self, name_or_id=None, resource_group=None, provider_namespace=None, parent=None,
                         resource_type=None):
        from argparse import Namespace
        ns = Namespace()
        ns.name_or_id = name_or_id
        ns.resource_group_name = resource_group
        ns.namespace = provider_namespace
        ns.parent = parent
        ns.resource_type = resource_type
        return ns

    @mock.patch('azure.cli.core.commands.client_factory.get_subscription_id', _mock_get_subscription_id)
    def test_monitor_resource_id(self):
        from azure.cli.command_modules.monitor.validators import get_target_resource_validator
        from azure.cli.core.mock import DummyCli

        cmd = mock.MagicMock()
        cmd.cli_ctx = DummyCli()
        validator = get_target_resource_validator('name_or_id', True)

        id = '/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/my-rg/providers/Microsoft.Compute/' \
             'virtualMachines/vm1'

        # must supply name or ID
        ns = self._build_namespace()
        with self.assertRaises(CLIError):
            validator(cmd, ns)

        # must only supply ID or name parameters
        ns = self._build_namespace(id, 'my-rg', 'blahblah', 'stuff')
        with self.assertRaises(CLIError):
            validator(cmd, ns)

        # error on invalid ID
        ns = self._build_namespace('bad-id')
        with self.assertRaises(CLIError):
            validator(cmd, ns)

        # allow Provider/Type syntax (same as resource commands)
        ns = self._build_namespace('vm1', 'my-rg', None, None, 'Microsoft.Compute/virtualMachines')
        validator(cmd, ns)
        self.assertEqual(ns.name_or_id, id)

        # allow Provider and Type separate
        ns = self._build_namespace('vm1', 'my-rg', 'Microsoft.Compute', None, 'virtualMachines')
        validator(cmd, ns)
        self.assertEqual(ns.name_or_id, id)

        # verify works with parent
        id = '/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/my-rg/providers/Microsoft.Compute/' \
             'fakeType/type1/anotherFakeType/type2/virtualMachines/vm1'
        ns = self._build_namespace('vm1', 'my-rg', 'Microsoft.Compute', 'fakeType/type1/anotherFakeType/type2',
                                   'virtualMachines')
        validator(cmd, ns)
        self.assertEqual(ns.name_or_id, id)

        # verify extra parameters are removed
        self.assertFalse(hasattr(ns, 'resource_name'))
        self.assertFalse(hasattr(ns, 'namespace'))
        self.assertFalse(hasattr(ns, 'parent'))
        self.assertFalse(hasattr(ns, 'resource_type'))


class MonitorMetricAlertActionTest(unittest.TestCase):

    def _build_namespace(self, name_or_id=None, resource_group=None, provider_namespace=None, parent=None,
                         resource_type=None):
        from argparse import Namespace
        return Namespace()

    def call_condition(self, ns, value):
        from azure.cli.command_modules.monitor.actions import MetricAlertConditionAction
        MetricAlertConditionAction('--condition', 'condition').__call__(None, ns, value.split(), '--condition')

    def check_condition(self, ns, time_aggregation, metric_namespace, metric_name, operator, threshold):
        prop = ns.condition[0]
        self.assertEqual(prop.time_aggregation, time_aggregation)
        self.assertEqual(prop.metric_name, metric_name)
        self.assertEqual(prop.operator, operator)
        self.assertEqual(prop.threshold, threshold)
        self.assertEqual(prop.metric_namespace, metric_namespace)

    def check_dimension(self, ns, index, name, operator, values):
        dim = ns.condition[0].dimensions[index]
        self.assertEqual(dim.name, name)
        self.assertEqual(dim.operator, operator)
        self.assertEqual(dim.values, values)

    def test_monitor_metric_alert_condition_action(self):

        from knack.util import CLIError

        ns = self._build_namespace()
        self.call_condition(ns, 'avg ns."CPU Percent" > 90')
        self.check_condition(ns, 'Average', 'ns', 'CPU Percent', 'GreaterThan', '90')

        ns = self._build_namespace()
        self.call_condition(ns, 'avg ns."a.b/c_d" > 90')
        self.check_condition(ns, 'Average', 'ns', 'a.b/c_d', 'GreaterThan', '90')

        ns = self._build_namespace()
        self.call_condition(ns, 'avg CPU Percent > 90')
        self.check_condition(ns, 'Average', None, 'CPU Percent', 'GreaterThan', '90')

        ns = self._build_namespace()
        self.call_condition(ns, 'avg "a.b/c_d" > 90')
        self.check_condition(ns, 'Average', None, 'a.b/c_d', 'GreaterThan', '90')

        ns = self._build_namespace()
        self.call_condition(ns, 'avg SuccessE2ELatency > 250 where ApiName includes GetBlob or PutBlob')
        self.check_condition(ns, 'Average', None, 'SuccessE2ELatency', 'GreaterThan', '250')
        self.check_dimension(ns, 0, 'ApiName', 'Include', ['GetBlob', 'PutBlob'])

        ns = self._build_namespace()
        self.call_condition(ns, 'avg ns.foo/bar_doo > 90')
        self.check_condition(ns, 'Average', 'ns', 'foo/bar_doo', 'GreaterThan', '90')

        ns = self._build_namespace()
        with self.assertRaisesRegexp(CLIError, 'usage error: --condition'):
            self.call_condition(ns, 'avg blah"what > 90')

        ns = self._build_namespace()
        with self.assertRaisesRegexp(CLIError, 'usage error: --condition'):
            self.call_condition(ns, 'avg Wra!!ga * woo')

        ns = self._build_namespace()
        self.call_condition(ns, 'avg SuccessE2ELatenc,|y > 250 where ApiName includes Get|,%_Blob or PutB,_lob')
        self.check_condition(ns, 'Average', None, 'SuccessE2ELatenc,|y', 'GreaterThan', '250')
        self.check_dimension(ns, 0, 'ApiName', 'Include', ['Get|,%_Blob', 'PutB,_lob'])


class MonitorAutoscaleActionTest(unittest.TestCase):

    def _build_namespace(self, name_or_id=None, resource_group=None, provider_namespace=None, parent=None,
                         resource_type=None):
        from argparse import Namespace
        return Namespace()

    def call_condition(self, ns, value):
        from azure.cli.command_modules.monitor.actions import AutoscaleConditionAction
        AutoscaleConditionAction('--condition', 'condition').__call__(None, ns, value.split(), '--condition')

    def check_condition(self, ns, metric_namespace, metric_name, operator, threshold, time_aggregation, time_window):
        prop = ns.condition
        self.assertEqual(prop.time_aggregation, time_aggregation)
        self.assertEqual(prop.metric_namespace, metric_namespace)
        self.assertEqual(prop.metric_name, metric_name)
        self.assertEqual(prop.operator, operator)
        self.assertEqual(prop.threshold, threshold)
        self.assertEqual(prop.time_window, time_window)

    def check_dimension(self, ns, index, dimension_name, operator, values):
        dim = ns.condition.dimensions[index]
        self.assertEqual(dim.dimension_name, dimension_name)
        self.assertEqual(dim.operator, operator)
        self.assertEqual(dim.values, values)

    def test_monitor_autoscale_condition_action(self):

        ns = self._build_namespace()
        self.call_condition(ns, '"ns" CPU Percent > 90 avg 1h5m')
        self.check_condition(ns, 'ns', 'CPU Percent', 'GreaterThan', '90', 'Average', 'PT5M1H')

        ns = self._build_namespace()
        self.call_condition(ns, 'CPU Percent > 90 avg 1h5m')
        self.check_condition(ns, None, 'CPU Percent', 'GreaterThan', '90', 'Average', 'PT5M1H')

        ns = self._build_namespace()
        self.call_condition(ns, 'process.cpu.usage > 0 avg 3m where App == app1 and Deployment == default and Instance == instance1')
        self.check_condition(ns, None, 'process.cpu.usage', 'GreaterThan', '0', 'Average', 'PT3M')
        self.check_dimension(ns, 0, 'App', 'Equals', ['app1'])
        self.check_dimension(ns, 1, 'Deployment', 'Equals', ['default'])
        self.check_dimension(ns, 2, 'Instance', 'Equals', ['instance1'])

        ns = self._build_namespace()
        self.call_condition(ns, '"Microsoft.AppPlatform/Spring" tomcat.global.request.total.count > 0 avg 3m where App == app1 or app3 and Deployment == default and Instance != instance1')
        self.check_condition(ns, "Microsoft.AppPlatform/Spring", 'tomcat.global.request.total.count', 'GreaterThan', '0', 'Average', 'PT3M')
        self.check_dimension(ns, 0, 'App', 'Equals', ['app1', 'app3'])
        self.check_dimension(ns, 1, 'Deployment', 'Equals', ['default'])
        self.check_dimension(ns, 2, 'Instance', 'NotEquals', ['instance1'])

        ns = self._build_namespace()
        self.call_condition(ns, '"kubernetes custom metrics %#*@_-" tomcat.global.request.total.count > 0 avg 3m where App == app1 or app3 and Deployment == default and Instance != instance1')
        self.check_condition(ns, "kubernetes custom metrics %#*@_-", 'tomcat.global.request.total.count', 'GreaterThan',
                             '0', 'Average', 'PT3M')
        self.check_dimension(ns, 0, 'App', 'Equals', ['app1', 'app3'])
        self.check_dimension(ns, 1, 'Deployment', 'Equals', ['default'])
        self.check_dimension(ns, 2, 'Instance', 'NotEquals', ['instance1'])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import sys
from importlib import import_module
import os

from logging import getLogger

logger = getLogger(__name__)

from knack.util import CLIError
from knack.help_files import helps
from azure.cli.core.mock import DummyCli
from azure.cli.core.util import get_installed_cli_distributions
from azure.cli.core._help import CliCommandHelpFile, CliGroupHelpFile
from azure.cli.core.file_util import _store_parsers, _is_group

try:
    from ruamel.yaml import YAML
    yaml = YAML()
    yaml.width = 1000 # prevents wrapping around in dumper.
    yaml.allow_duplicate_keys = True # TODO: allow duplicate keys within help entries. see az container create. Remove this.
except ImportError as e:
    msg = "{}\npip install ruamel.Yaml to use this script.".format(e)
    exit(msg)

PACKAGE_PREFIX = "azure.cli.command_modules"
CLI_PACKAGE_NAME = 'azure-cli'
COMPONENT_PREFIX = 'azure-cli-'

failed = 0

loaded_helps = {}

def get_all_help(cli_ctx):
    invoker = cli_ctx.invocation
    help_ctx = cli_ctx.help_cls(cli_ctx)
    if not invoker:
        raise CLIError('CLI context does not contain invocation.')

    parser_keys = []
    parser_values = []
    sub_parser_keys = []
    sub_parser_values = []
    _store_parsers(invoker.parser, parser_keys, parser_values, sub_parser_keys, sub_parser_values)
    for cmd, parser in zip(parser_keys, parser_values):
        if cmd not in sub_parser_keys:
            sub_parser_keys.append(cmd)
            sub_parser_values.append(parser)
    help_files = []
    for cmd, parser in zip(sub_parser_keys, sub_parser_values):
        if cmd in loaded_helps:
            try:
                help_file = CliGroupHelpFile(help_ctx, cmd, parser) if _is_group(parser) \
                    else CliCommandHelpFile(help_ctx, cmd, parser)
                help_file.load(parser)
                help_files.append(help_file)
            except Exception as ex:  # pylint: disable=broad-except
                print("Skipped '{}' due to '{}'".format(cmd, ex))
    help_files = sorted(help_files, key=lambda x: x.command)
    assert {help_file.command for help_file in help_files} == set(loaded_helps.keys())
    print("Loaded {} help files".format(len(help_files)))
    return help_files


def create_invoker_and_load_cmds_and_args(cli_ctx):
    global loaded_helps
    from knack import events
    from azure.cli.core.commands import register_cache_arguments
    from azure.cli.core.commands.arm import register_global_subscription_argument, register_ids_argument

    invoker = cli_ctx.invocation_cls(cli_ctx=cli_ctx, commands_loader_cls=cli_ctx.commands_loader_cls,
                                     parser_cls=cli_ctx.parser_cls, help_cls=cli_ctx.help_cls)
    cli_ctx.invocation = invoker
    invoker.commands_loader.skip_applicability = True
    temp_help = helps.copy()
    invoker.commands_loader.load_command_table(None) # this ends up loading all the helpfiles, which could be problematic with duplicate key commands like acs create
    helps.clear()
    helps.update(temp_help)

    # turn off applicability check for applicable loaders
    new_cmd_to_loader_map = {}
    new_command_group_table = {}
    new_command_table = {}
    for cmd in loaded_helps.keys():
        if cmd in invoker.commands_loader.cmd_to_loader_map: # if a command from help, then update commands_loader
            new_cmd_to_loader_map[cmd] = invoker.commands_loader.cmd_to_loader_map[cmd]
            new_command_table[cmd] = invoker.commands_loader.command_table[cmd]
        else: # else a group then update
            new_command_group_table[cmd] = invoker.commands_loader.command_group_table[cmd]

    # include commands of groups in help.py, even though the commands themselves might not be in help.py, so their subparsers can be added.
    for cmd in new_command_group_table.keys():
        all_cmds = invoker.commands_loader.cmd_to_loader_map.keys()
        for old_cmd in all_cmds:
            if old_cmd.startswith(cmd) and old_cmd != cmd:
                new_cmd_to_loader_map[old_cmd] = invoker.commands_loader.cmd_to_loader_map[old_cmd]
                new_command_table[old_cmd] = invoker.commands_loader.command_table[old_cmd]

    invoker.commands_loader.cmd_to_loader_map = new_cmd_to_loader_map
    invoker.commands_loader.command_table = new_command_table
    invoker.commands_loader.command_group_table = new_command_group_table

    for loaders in invoker.commands_loader.cmd_to_loader_map.values():
        for loader in loaders:
            loader.skip_applicability = True

    for command in invoker.commands_loader.command_table:
        invoker.commands_loader.load_arguments(command)

    assert len(new_command_table) == len(new_cmd_to_loader_map)
    assert set(list(new_command_table.keys()) + list(new_command_group_table.keys())) >= set(loaded_helps.keys())

    register_global_subscription_argument(cli_ctx)
    register_ids_argument(cli_ctx)  # global subscription must be registered first!
    register_cache_arguments(cli_ctx)
    cli_ctx.raise_event(events.EVENT_INVOKER_POST_CMD_TBL_CREATE, commands_loader=invoker.commands_loader)
    invoker.parser.load_command_table(invoker.commands_loader)

# this must be called before loading any command modules. Otherwise helps object will have every help.py file's contents
def convert(target_mod_or_file, mod_name, test=False):
    global loaded_helps

    if os.path.exists(target_mod_or_file):
        out_file = target_mod_or_file
        target_mod = import_module("{}.{}._help".format(PACKAGE_PREFIX, mod_name))

    else: # else not a file, but a mod.
        try:
            target_mod = import_module(target_mod_or_file)
        except ModuleNotFoundError as e: # azure.cli.command_modules.core
            logger.warning(e)
            return None, None
        loader_file_path = os.path.abspath(target_mod.__file__)
        out_file = os.path.join(os.path.dirname(loader_file_path), "help.yaml")

    if test and os.path.exists(out_file):
        print("{}/help.yaml already exists. Will remove and rewrite: {}\n".format(mod_name, out_file))
        os.remove(out_file)

    # the modules keys are keys added to helps object from fresh import....
    help_dict = target_mod.helps
    result = _get_new_yaml_dict(help_dict)

    # clear modules help from knack.helps, store help.py info
    for key, value in help_dict.items():
        loaded_helps[key] = value

    return out_file, result

def delete(target_mods):
    for mod_name in target_mods:
        try:
            target_mod = import_module(mod_name)
        except ModuleNotFoundError as e:  # azure.cli.command_modules.core
            print(e)
            print("SKIPPING...")
            continue
        loader_file_path = os.path.abspath(target_mod.__file__)
        yaml_file = os.path.join(os.path.dirname(loader_file_path), "help.yaml")
        if os.path.exists(yaml_file):
            print("Removing {}".format(yaml_file))
            os.remove(yaml_file)

def get_all_mod_names():
    installed_dists = get_installed_cli_distributions()
    mod_name_list = list(sorted(dist.key.replace(COMPONENT_PREFIX, '') for dist in installed_dists if dist.key.startswith(COMPONENT_PREFIX)))
    return mod_name_list


def _get_new_yaml_dict(help_dict):

    result = dict(version=1, content=[])
    content = result['content']

    for command_or_group, yaml_text in help_dict.items():
        help_dict = yaml.safe_load(yaml_text)

        type = help_dict["type"]

        elem = {type: dict(name=command_or_group)}
        elem_content = elem[type]

        _convert_summaries(old_dict=help_dict, new_dict=elem_content)

        if "parameters" in help_dict:
            parameters = []
            for param in help_dict["parameters"]:
                new_param = dict()
                if "name" in param:
                    options = param["name"].split()
                    new_param["name"] = max(options, key=lambda x: len(x))
                _convert_summaries(old_dict=param, new_dict=new_param)

                if "populator-commands" in param:
                    new_param["value-sources"] = []
                    for item in param["populator-commands"]:
                        new_param["value-sources"].append({"link": {"command" : item}})
                parameters.append(new_param)
            elem_content["arguments"] = parameters

        if "examples" in help_dict:
            elem_examples = []
            for ex in help_dict["examples"]:
                new_ex = dict()
                if "name" in ex:
                    new_ex["summary"] = ex["name"]
                if "text" in ex:
                    new_ex["command"] = ex["text"]
                supported_profiles, unsupported_profiles = "supported-profiles", "unsupported-profiles"
                if supported_profiles in ex:
                    new_ex[supported_profiles] = ex[supported_profiles]
                if unsupported_profiles in ex:
                    new_ex[unsupported_profiles] = ex[unsupported_profiles]
                elem_examples.append(new_ex)
            elem_content["examples"] = elem_examples

        content.append(elem)

    return result


def _convert_summaries(old_dict, new_dict):
    if "short-summary" in old_dict:
        new_dict["summary"] = old_dict["short-summary"]
    if "long-summary" in old_dict:
        new_dict["description"] = old_dict["long-summary"]

def assert_help_objs_equal(old_help, new_help):
    assert_true_or_warn(old_help.name, new_help.name)
    assert_true_or_warn(old_help.type, new_help.type)
    assert_true_or_warn(old_help.short_summary, new_help.short_summary)
    assert_true_or_warn(old_help.long_summary, new_help.long_summary)
    assert_true_or_warn(old_help.command, new_help.command)

    old_examples = sorted(old_help.examples, key=lambda x: x.short_summary)
    new_examples = sorted(new_help.examples, key=lambda x: x.short_summary)
    assert_true_or_warn(len(old_examples), len(new_examples))
    # note: this cannot test if min / max version were added as these fields weren't stored in helpfile objects previously.
    for old_ex, new_ex in zip(old_examples, new_examples):
        assert_true_or_warn (old_ex.short_summary, new_ex.short_summary)
        assert_true_or_warn (old_ex.command, new_ex.command)
        assert_true_or_warn (old_ex.long_summary, new_ex.long_summary)

    # group and not command, we are done checking.
    if old_help.type == "group":
        return

    old_parameters = sorted(old_help.parameters, key=lambda x: x.name_source)
    new_parameters = sorted(new_help.parameters, key=lambda x: x.name_source)
    assert_true_or_warn(len(old_parameters), len(new_parameters))
    assert_params_equal(old_parameters, new_parameters)


def assert_params_equal(old_parameters, new_parameters):
    for old, new in zip(old_parameters, new_parameters):
        assert_true_or_warn(old.short_summary, new.short_summary)
        assert_true_or_warn(old.long_summary, new.long_summary)
        assert_true_or_warn(old.value_sources, new.value_sources)


def assert_true_or_warn(x, y):
    try:
        assert x == y

    except AssertionError:
        # if is list try to find exactly where there is failure
        if isinstance(x, list) and len(x) == len(y):
            for x_1, y_1 in zip(x, y):
                assert_true_or_warn(x_1, y_1)
        else:
            msg = "\nvalues:\n\n{}\n\nand\n\n{}\n\nare not equal.\n".format(x, y)

            global failed
            failed+=1
            if failed:
                msg = "{}{}".format(msg, "-------------------\nAssertion Failed!!!\nExiting test.\n-------------------\n")
                exit(msg)


if __name__ == "__main__":
    if sys.version_info[0] < 3:
        raise Exception("This script requires Python 3")

    args = [arg.strip() for arg in sys.argv[1:]]
    test = False

    msg = 'Usage: python help_convert.py (MOD | --test | --delete | --all | MOD --test | path/help.py --package {myext.package})\n'

    if "--help" in args or "-h" in args:
        print(msg)
        exit(0)

    if len(args) > 3 or len(args) == 0:
        exit(msg)

    if len(args) == 2:
        if args[1].lower() != "--test":
            exit(msg)
        else:
            test = True
    if len(args) == 3:
        if not args[0].lower().endswith("_help.py") or args[1].lower() != "--package":
            exit(msg)
        else:
            raise NotImplementedError

    target_mods = None
    # if args[0].lower() in ["--test", "--all"]:
    #     # convert all modules and test
    #     mod_names = get_all_mod_names()
    #     target_mods = ["{}.{}._help".format(PACKAGE_PREFIX, mod) for mod in mod_names]
    #     if args[0].lower() == "--test":
    #         test = True
    if args[0].lower() == "--delete":
        mod_names = get_all_mod_names()
        mod_names.append("sqlvm")
        target_mods = ["{}.{}._help".format(PACKAGE_PREFIX, mod) for mod in mod_names]
        delete(target_mods)
        exit(0)
    elif args[0].lower() == "--get-all-mods":
        mod_names = get_all_mod_names()
        with open("mod.txt", "w") as f:
            for name in mod_names:
                f.write(name + "\n")
        exit(0)
    else:
        mod_names = [args[0]]
        # attempt to find and load the desired module.
        if "_help.py" not in mod_names[0]:
            target_mods = ["{}.{}._help".format(PACKAGE_PREFIX, mod_names[0])]
        else:
            target_mods = [mod_names[0]]
            mod_names = [os.path.split(os.path.split(mod_names[0])[0])[1]] # if help.py, get mod name

    if test:
        # convert _help.py contents to help.yaml. Write out help.yaml
        print("Generating new help.yaml file contents. Holding off on writing contents...")
        file_to_help = {}

        for mod, mod_name in zip(target_mods, mod_names):
            file, help_dict = convert(mod, mod_name, test=True)
            if file is None or help_dict is None:
                continue
            file_to_help[file] = help_dict

        if not file_to_help:
            exit("No help.yaml file generated for {}.".format(", ".join(mod_names)))

        print("Loading Commands...")
        # setup CLI
        az_cli = DummyCli()
        create_invoker_and_load_cmds_and_args(az_cli)

        print("Loading all old help...")
        all_cli_help = get_all_help(az_cli)
        old_loaded_help = {data.command: data for data in all_cli_help}

        print("Now writing out new help.yaml file contents...")
        for file, help_dict in file_to_help.items():
            logger.warning("Writing {} ...".format(file))
            with open(file, "w") as f:
                yaml.safe_dump(help_dict, f)

        print("Loading all help again...")
        all_cli_help = get_all_help(az_cli)
        new_loaded_help = {data.command: data for data in all_cli_help}

        assert len(old_loaded_help) == len(new_loaded_help)

        diff_dict = {}
        for command in old_loaded_help:
            diff_dict[command] = (old_loaded_help[command], new_loaded_help[command])

        logger.warning("Loaded {} help objects".format(len(new_loaded_help)))
        logger.warning("Verifying that help objects are the same for _help.py and help.yaml.")
        assert len(diff_dict) == len(loaded_helps)
        for old, new in diff_dict.values():
            assert_help_objs_equal(old, new)

    else:
        if len(target_mods) == 1:
            print("Generating help.yaml file...")
            out_file, result = convert(target_mods[0], mod_names[0])
            with open(out_file, "w") as f:
                yaml.safe_dump(result, f)
            print("Done! Successfully generated {0}/help.yaml in {0} module.".format(mod_names[0]))
        else:
            print("Generating help.yaml files...")
            for mod, mod_name in zip(target_mods, mod_names):
                out_file, result = convert(mod, mod_name)
                if out_file is None or result is None:
                    continue
                with open(out_file, "w") as f:
                    yaml.safe_dump(result, f)
                print("Successfully generated {0}/help.yaml in {0} module.".format(mod_name))




# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import json
import os
import time
import mock
import unittest

from azure_devtools.scenario_tests.const import MOCKED_SUBSCRIPTION_ID
from azure_devtools.scenario_tests import AllowLargeResponse
from azure.cli.testsdk import ScenarioTest, LiveScenarioTest, ResourceGroupPreparer, create_random_name, live_only, record_only
from azure.cli.core.util import get_file_json


class ResourceGroupScenarioTest(ScenarioTest):

    @ResourceGroupPreparer(name_prefix='cli_test_rg_scenario')
    def test_resource_group(self, resource_group):

        self.cmd('group delete -n {rg} --yes')
        self.cmd('group exists -n {rg}',
                 checks=self.check('@', False))

        self.cmd('group create -n {rg} -l westus --tag a=b c', checks=[
            self.check('name', '{rg}'),
            self.check('tags', {'a': 'b', 'c': ''})
        ])
        self.cmd('group exists -n {rg}',
                 checks=self.check('@', True))
        self.cmd('group show -n {rg}', checks=[
            self.check('name', '{rg}'),
            self.check('tags', {'a': 'b', 'c': ''})
        ])
        self.cmd('group list --tag a=b', checks=[
            self.check('[0].name', '{rg}'),
            self.check('[0].tags', {'a': 'b', 'c': ''})
        ])
        # test --force-string
        self.kwargs.update({'tag': "\"{\\\"k\\\":\\\"v\\\"}\""})
        self.cmd('group update -g {rg} --tags ""',
                 checks=self.check('tags', {}))
        self.cmd('group update -g {rg} --set tags.a={tag}',
                 checks=self.check('tags.a', "{{'k': 'v'}}"))
        self.cmd('group update -g {rg} --set tags.b={tag} --force-string',
                 checks=self.check('tags.b', '{{\"k\":\"v\"}}'))


class ResourceGroupNoWaitScenarioTest(ScenarioTest):

    @ResourceGroupPreparer(name_prefix='cli_rg_nowait_test')
    def test_resource_group_no_wait(self, resource_group):

        self.cmd('group delete -n {rg} --no-wait --yes',
                 checks=self.is_empty())
        self.cmd('group wait --deleted -n {rg}',
                 checks=self.is_empty())
        self.cmd('group exists -n {rg}',
                 checks=self.check('@', False))
        self.cmd('group create -n {rg} -l westus',
                 checks=self.check('name', '{rg}'))
        self.cmd('group exists -n {rg}',
                 checks=self.check('@', True))
        self.cmd('group wait --exists -n {rg}',
                 checks=self.is_empty())


class ResourceScenarioTest(ScenarioTest):

    @ResourceGroupPreparer(name_prefix='cli_test_resource_scenario', location='southcentralus')
    @AllowLargeResponse()
    def test_resource_scenario(self, resource_group, resource_group_location):
        self.kwargs.update({
            'loc': resource_group_location,
            'vnet': self.create_random_name('vnet-', 30),
            'subnet': self.create_random_name('subnet-', 30),
            'rt': 'Microsoft.Network/virtualNetworks'
        })
        vnet_count = self.cmd("resource list --query \"length([?name=='{vnet}'])\"").get_output_in_json() or 0
        self.cmd('network vnet create -g {rg} -n {vnet} --subnet-name {subnet} --tags cli-test=test')
        vnet_count += 1

        self.cmd('resource list',
                 checks=self.check("length([?name=='{vnet}'])", vnet_count))
        self.cmd('resource list -l {loc}',
                 checks=self.check("length([?location == '{loc}']) == length(@)", True))
        self.cmd('resource list --resource-type {rt}',
                 checks=self.check("length([?name=='{vnet}'])", vnet_count))
        self.cmd('resource list --name {vnet}', checks=[
            self.check("length([?name=='{vnet}'])", vnet_count),
            self.check('[0].provisioningState', 'Succeeded')
        ])
        self.cmd('resource list --tag cli-test',
                 checks=self.check("length([?name=='{vnet}'])", vnet_count))
        self.cmd('resource list --tag cli-test=test',
                 checks=self.check("length([?name=='{vnet}'])", vnet_count))

        # check for simple resource with tag
        self.cmd('resource show -n {vnet} -g {rg} --resource-type Microsoft.Network/virtualNetworks', checks=[
            self.check('name', '{vnet}'),
            self.check('location', '{loc}'),
            self.check('resourceGroup', '{rg}'),
            self.check('tags', {'cli-test': 'test'})
        ])
        # check for child resource
        self.cmd('resource show -n {subnet} -g {rg} --namespace Microsoft.Network --parent virtualNetworks/{vnet} --resource-type subnets', checks=[
            self.check('name', '{subnet}'),
            self.check('resourceGroup', '{rg}')
        ])

        # clear tag and verify
        self.cmd('resource tag -n {vnet} -g {rg} --resource-type Microsoft.Network/virtualNetworks --tags')
        self.cmd('resource show -n {vnet} -g {rg} --resource-type Microsoft.Network/virtualNetworks',
                 checks=self.check('tags', {}))

        # delete and verify
        self.cmd('resource delete -n {vnet} -g {rg} --resource-type {rt}')
        time.sleep(10)
        self.cmd('resource list', checks=self.check("length([?name=='{vnet}'])", 0))


class ResourceIDScenarioTest(ScenarioTest):

    @ResourceGroupPreparer(name_prefix='cli_test_resource_id')
    def test_resource_id_scenario(self, resource_group):

        self.kwargs.update({
            'vnet': 'cli_test_resource_id_vnet',
            'subnet': 'cli_test_resource_id_subnet'
        })

        self.cmd('network vnet create -g {rg} -n {vnet} --subnet-name {subnet}')

        self.kwargs['sub'] = self.get_subscription_id()

        self.kwargs['vnet_id'] = '/subscriptions/{sub}/resourceGroups/{rg}/providers/Microsoft.Network/virtualNetworks/{vnet}'.format(
            **self.kwargs)
        self.cmd('resource tag --id {vnet_id} --tags tag-vnet')
        self.cmd('resource show --id {vnet_id}', checks=[
            self.check('name', '{vnet}'),
            self.check('resourceGroup', '{rg}'),
            self.check('tags', {'tag-vnet': ''})
        ])

        self.kwargs['subnet_id'] = '/subscriptions/{sub}/resourceGroups/{rg}/providers/Microsoft.Network/virtualNetworks/{vnet}/subnets/{subnet}'.format(
            **self.kwargs)
        self.cmd('resource show --id {subnet_id}', checks=[
            self.check('name', '{subnet}'),
            self.check('resourceGroup', '{rg}'),
            self.check('properties.addressPrefix', '10.0.0.0/24')
        ])

        self.cmd('resource update --id {subnet_id} --set properties.addressPrefix=10.0.0.0/22',
                 checks=self.check('properties.addressPrefix', '10.0.0.0/22'))

        self.cmd('resource delete --id {subnet_id}', checks=self.is_empty())
        self.cmd('resource delete --id {vnet_id}', checks=self.is_empty())


class ResourceGenericUpdate(LiveScenarioTest):
    @ResourceGroupPreparer(name_prefix='cli_test_resource_id')
    def test_resource_id_scenario(self, resource_group):
        self.kwargs.update({
            'stor_1': self.create_random_name(prefix='stor1', length=10),
            'stor_2': self.create_random_name(prefix='stor2', length=10)
        })

        # create storage accounts
        self.cmd('az storage account create -g {rg} -n {stor_1}')
        self.cmd('az storage account create -g {rg} -n {stor_2}')

        # get ids
        self.kwargs['stor_ids'] = " ".join(self.cmd('az storage account list -g {rg} --query "[].id"').get_output_in_json())

        # update tags
        self.cmd('az storage account update --ids {stor_ids} --set tags.isTag=True tags.isNotTag=False')

        self.cmd('az storage account show --name {stor_1} -g {rg}', checks=[
            self.check('tags.isTag', 'True'),
            self.check('tags.isNotTag', 'False')
        ])
        self.cmd('az storage account show --name {stor_2} -g {rg}', checks=[
            self.check('tags.isTag', 'True'),
            self.check('tags.isNotTag', 'False')
        ])

        # delete tags.isTag
        self.cmd('az storage account update --ids {stor_ids} --remove tags.isTag')

        self.cmd('az storage account show --name {stor_1} -g {rg} --query "tags"', checks=[
            self.check('isNotTag', 'False'),
            self.check('isTag', None)
        ])
        self.cmd('az storage account show --name {stor_2} -g {rg} --query "tags"', checks=[
            self.check('isNotTag', 'False'),
            self.check('isTag', None)
        ])

        # delete tags.isNotTag
        self.cmd('az storage account update --ids {stor_ids} --remove tags.isNotTag')

        # check tags is empty.
        self.cmd('az storage account show --name {stor_1} -g {rg} --query "tags"', checks=self.is_empty())
        self.cmd('az storage account show --name {stor_2} -g {rg} --query "tags"', checks=self.is_empty())


class ResourceCreateAndShowScenarioTest(ScenarioTest):

    @ResourceGroupPreparer(name_prefix='cli_test_resource_create')
    def test_resource_create_and_show(self, resource_group, resource_group_location):

        self.kwargs.update({
            'plan': 'cli_res_create_plan',
            'app': 'clirescreateweb',
            'loc': resource_group_location
        })

        self.cmd('resource create -g {rg} -n {plan} --resource-type Microsoft.web/serverFarms --is-full-object --properties "{{\\"location\\":\\"{loc}\\",\\"sku\\":{{\\"name\\":\\"B1\\",\\"tier\\":\\"BASIC\\"}}}}"',
                 checks=self.check('name', '{plan}'))

        result = self.cmd('resource create -g {rg} -n {app} --resource-type Microsoft.web/sites --properties "{{\\"serverFarmId\\":\\"{plan}\\"}}"',
                          checks=self.check('name', '{app}')).get_output_in_json()

        self.kwargs['app_settings_id'] = result['id'] + '/config/appsettings'
        self.kwargs['app_config_id'] = result['id'] + '/config/web'
        self.cmd('resource create --id {app_settings_id} --properties "{{\\"key2\\":\\"value12\\"}}"',
                 checks=[self.check('properties.key2', 'value12')])

        self.cmd('resource show --id {app_config_id}',
                 checks=self.check('properties.publishingUsername', '${app}'))
        self.cmd('resource show --id {app_config_id} --include-response-body',
                 checks=self.check('responseBody.properties.publishingUsername', '${app}'))


class TagScenarioTest(ScenarioTest):

    def test_tag_scenario(self):

        self.kwargs.update({
            'tag': 'cli_test_tag'
        })

        tag_values = self.cmd('tag list --query "[?tagName == \'{tag}\'].values[].tagValue"').get_output_in_json()
        for tag_value in tag_values:
            self.cmd('tag remove-value --value {} -n {{tag}}'.format(tag_value))
        self.cmd('tag delete -n {tag} -y')

        self.cmd('tag list --query "[?tagName == \'{tag}\']"', checks=self.is_empty())
        self.cmd('tag create -n {tag}', checks=[
            self.check('tagName', '{tag}'),
            self.check('values', []),
            self.check('count.value', 0)
        ])
        self.cmd('tag add-value -n {tag} --value test')
        self.cmd('tag add-value -n {tag} --value test2')
        self.cmd('tag list --query "[?tagName == \'{tag}\']"',
                 checks=self.check('[].values[].tagValue', [u'test', u'test2']))
        self.cmd('tag remove-value -n {tag} --value test')
        self.cmd('tag list --query "[?tagName == \'{tag}\']"',
                 checks=self.check('[].values[].tagValue', [u'test2']))
        self.cmd('tag remove-value -n {tag} --value test2')
        self.cmd('tag list --query "[?tagName == \'{tag}\']"',
                 checks=self.check('[].values[].tagValue', []))
        self.cmd('tag delete -n {tag} -y')
        self.cmd('tag list --query "[?tagName == \'{tag}\']"',
                 checks=self.is_empty())


class ProviderRegistrationTest(ScenarioTest):

    def test_provider_registration(self):

        self.kwargs.update({'prov': 'TrendMicro.DeepSecurity'})
        result = self.cmd('provider show -n {prov}').get_output_in_json()
        if result['registrationState'] == 'Unregistered':
            self.cmd('provider register -n {prov}')
            result = self.cmd('provider show -n {prov}').get_output_in_json()
            self.assertTrue(result['registrationState'] in ['Registering', 'Registered'])
            self.cmd('provider unregister -n {prov}')
            result = self.cmd('provider show -n {prov}').get_output_in_json()
            self.assertTrue(result['registrationState'] in ['Unregistering', 'Unregistered'])
        else:
            self.cmd('provider unregister -n {prov}')
            result = self.cmd('provider show -n {prov}').get_output_in_json()
            self.assertTrue(result['registrationState'] in ['Unregistering', 'Unregistered'])
            self.cmd('provider register -n {prov}')
            result = self.cmd('provider show -n {prov}').get_output_in_json()
            self.assertTrue(result['registrationState'] in ['Registering', 'Registered'])


class ProviderOperationTest(ScenarioTest):

    def test_provider_operation(self):
        self.cmd('provider operation show --namespace microsoft.compute', checks=[
            self.check('id', '/providers/Microsoft.Authorization/providerOperations/Microsoft.Compute'),
            self.check('type', 'Microsoft.Authorization/providerOperations')
        ])
        self.cmd('provider operation show --namespace microsoft.compute', checks=[
            self.check('id', '/providers/Microsoft.Authorization/providerOperations/Microsoft.Compute'),
            self.check('type', 'Microsoft.Authorization/providerOperations')
        ])
        self.cmd('provider operation show --namespace microsoft.storage', checks=[
            self.check('id', '/providers/Microsoft.Authorization/providerOperations/Microsoft.Storage'),
            self.check('type', 'Microsoft.Authorization/providerOperations')
        ])


class SubscriptionLevelDeploymentTest(LiveScenarioTest):
    def tearDown(self):
        self.cmd('policy assignment delete -n location-lock')
        self.cmd('policy definition delete -n policy2')

    def test_subscription_level_deployment(self):
        curr_dir = os.path.dirname(os.path.realpath(__file__))
        self.kwargs.update({
            'tf': os.path.join(curr_dir, 'subscription_level_template.json').replace('\\', '\\\\'),
            'params': os.path.join(curr_dir, 'subscription_level_parameters.json').replace('\\', '\\\\'),
            # params-uri below is the raw file url of the subscription_level_parameters.json above
            'params_uri': 'https://raw.githubusercontent.com/Azure/azure-cli/dev/src/command_modules/azure-cli-resource/azure/cli/command_modules/resource/tests/latest/subscription_level_parameters.json',
            'dn': self.create_random_name('azure-cli-sub-level-desubscription_level_parametersployment', 40)
        })

        self.cmd('group create --name cli_test_subscription_level_deployment --location WestUS', checks=[
            self.check('properties.provisioningState', 'Succeeded')
        ])

        self.cmd('deployment validate --location WestUS --template-file "{tf}" --parameters @"{params}"', checks=[
            self.check('properties.provisioningState', 'Succeeded')
        ])

        self.cmd('deployment validate --location WestUS --template-file "{tf}" --parameters @"{params_uri}"', checks=[
            self.check('properties.provisioningState', 'Succeeded')
        ])

        self.cmd('deployment create -n {dn} --location WestUS --template-file "{tf}" --parameters @"{params}"', checks=[
            self.check('properties.provisioningState', 'Succeeded'),
        ])

        self.cmd('deployment list', checks=[
            self.check('[0].name', '{dn}'),
        ])

        self.cmd('deployment show -n {dn}', checks=[
            self.check('name', '{dn}')
        ])

        self.cmd('deployment export -n {dn}', checks=[
        ])

        self.cmd('deployment operation list -n {dn}', checks=[
            self.check('length([])', 4)
        ])


class DeploymentTest(ScenarioTest):

    @ResourceGroupPreparer(name_prefix='cli_test_deployment_lite')
    def test_group_deployment_lite(self, resource_group):
        # ensures that a template that is missing "parameters" or "resources" still deploys
        curr_dir = os.path.dirname(os.path.realpath(__file__))

        self.kwargs.update({
            'tf': os.path.join(curr_dir, 'test-template-lite.json').replace('\\', '\\\\'),
            'dn': self.create_random_name('azure-cli-deployment', 30)
        })

        self.cmd('group deployment create -g {rg} -n {dn} --template-file "{tf}"', checks=[
            self.check('properties.provisioningState', 'Succeeded'),
            self.check('resourceGroup', '{rg}')
        ])

    @ResourceGroupPreparer(name_prefix='cli_test_deployment')
    def test_group_deployment(self, resource_group):
        curr_dir = os.path.dirname(os.path.realpath(__file__))
        self.kwargs.update({
            'tf': os.path.join(curr_dir, 'test-template.json').replace('\\', '\\\\'),
            'params': os.path.join(curr_dir, 'test-params.json').replace('\\', '\\\\'),
            # params-uri below is the raw file url of the test_params.json above
            'params_uri': 'https://raw.githubusercontent.com/Azure/azure-cli/dev/src/azure-cli/azure/cli/command_modules/resource/tests/latest/test-params.json',
            'of': os.path.join(curr_dir, 'test-object.json').replace('\\', '\\\\'),
            'dn': 'azure-cli-deployment'
        })
        self.kwargs['subnet_id'] = self.cmd('network vnet create -g {rg} -n vnet1 --subnet-name subnet1').get_output_in_json()['newVNet']['subnets'][0]['id']

        self.cmd('group deployment validate -g {rg} --template-file "{tf}" --parameters @"{params}" --parameters subnetId="{subnet_id}" --parameters backendAddressPools=@"{of}"', checks=[
            self.check('properties.provisioningState', 'Succeeded')
        ])

        self.cmd('group deployment validate -g {rg} --template-file "{tf}" --parameters "{params_uri}" --parameters subnetId="{subnet_id}" --parameters backendAddressPools=@"{of}"', checks=[
            self.check('properties.provisioningState', 'Succeeded')
        ])

        self.cmd('group deployment create -g {rg} -n {dn} --template-file "{tf}" --parameters @"{params}" --parameters subnetId="{subnet_id}" --parameters backendAddressPools=@"{of}"', checks=[
            self.check('properties.provisioningState', 'Succeeded'),
            self.check('resourceGroup', '{rg}')
        ])
        self.cmd('network lb show -g {rg} -n test-lb',
                 checks=self.check('tags', {'key': 'super=value'}))

        self.cmd('group deployment list -g {rg}', checks=[
            self.check('[0].name', '{dn}'),
            self.check('[0].resourceGroup', '{rg}')
        ])
        self.cmd('group deployment show -g {rg} -n {dn}', checks=[
            self.check('name', '{dn}'),
            self.check('resourceGroup', '{rg}')
        ])
        self.cmd('group deployment operation list -g {rg} -n {dn}', checks=[
            self.check('length([])', 2),
            self.check('[0].resourceGroup', '{rg}')
        ])

    @ResourceGroupPreparer(name_prefix='cli_test_on_error_deployment_lastsuccessful')
    def test_group_on_error_deployment_lastsuccessful(self, resource_group):
        curr_dir = os.path.dirname(os.path.realpath(__file__))

        self.kwargs.update({
            'tf': os.path.join(curr_dir, 'test-template-lite.json').replace('\\', '\\\\'),
            'dn': self.create_random_name('azure-cli-deployment', 30),
            'onErrorType': 'LastSuccessful',
            'sdn': self.create_random_name('azure-cli-deployment', 30)
        })

        self.cmd('group deployment create -g {rg} -n {dn} --template-file "{tf}"', checks=[
            self.check('properties.provisioningState', 'Succeeded'),
            self.check('resourceGroup', '{rg}'),
            self.check('properties.onErrorDeployment', None)
        ])

        self.cmd('group deployment create -g {rg} -n {sdn} --template-file "{tf}" --rollback-on-error', checks=[
            self.check('properties.provisioningState', 'Succeeded'),
            self.check('resourceGroup', '{rg}'),
            self.check('properties.onErrorDeployment.deploymentName', '{dn}'),
            self.check('properties.onErrorDeployment.type', '{onErrorType}')
        ])

    @ResourceGroupPreparer(name_prefix='cli_test_on_error_deployment_specificdeployment')
    def test_group_on_error_deployment_specificdeployment(self, resource_group):
        curr_dir = os.path.dirname(os.path.realpath(__file__))

        self.kwargs.update({
            'tf': os.path.join(curr_dir, 'test-template-lite.json').replace('\\', '\\\\'),
            'dn': self.create_random_name('azure-cli-deployment', 30),
            'onErrorType': 'SpecificDeployment',
            'sdn': self.create_random_name('azure-cli-deployment', 30)
        })

        self.cmd('group deployment create -g {rg} -n {dn} --template-file "{tf}"', checks=[
            self.check('properties.provisioningState', 'Succeeded'),
            self.check('resourceGroup', '{rg}'),
            self.check('properties.onErrorDeployment', None)
        ])

        self.cmd('group deployment create -g {rg} -n {sdn} --template-file "{tf}" --rollback-on-error {dn}', checks=[
            self.check('properties.provisioningState', 'Succeeded'),
            self.check('resourceGroup', '{rg}'),
            self.check('properties.onErrorDeployment.deploymentName', '{dn}'),
            self.check('properties.onErrorDeployment.type', '{onErrorType}')
        ])


class DeploymentLiveTest(LiveScenarioTest):
    @ResourceGroupPreparer()
    def test_group_deployment_progress(self, resource_group):
        from azure.cli.testsdk.utilities import force_progress_logging
        curr_dir = os.path.dirname(os.path.realpath(__file__))

        self.kwargs.update({
            'tf': os.path.join(curr_dir, 'test-template.json').replace('\\', '\\\\'),
            'params': os.path.join(curr_dir, 'test-params.json').replace('\\', '\\\\'),
            'of': os.path.join(curr_dir, 'test-object.json').replace('\\', '\\\\'),
            'dn': 'azure-cli-deployment2'
        })

        self.kwargs['subnet_id'] = self.cmd('network vnet create -g {rg} -n vnet1 --subnet-name subnet1').get_output_in_json()['newVNet']['subnets'][0]['id']

        with force_progress_logging() as test_io:
            self.cmd('group deployment create --verbose -g {rg} -n {dn} --template-file "{tf}" --parameters @"{params}" --parameters subnetId="{subnet_id}" --parameters backendAddressPools=@"{of}"')

        # very the progress
        lines = test_io.getvalue().splitlines()
        for line in lines:
            self.assertTrue(line.split(':')[0] in ['Accepted', 'Succeeded'])
        self.assertTrue('Succeeded: {} (Microsoft.Resources/deployments)'.format(self.kwargs['dn']), lines)


class DeploymentNoWaitTest(ScenarioTest):

    @ResourceGroupPreparer(name_prefix='cli_test_group_deployment_no_wait')
    def test_group_deployment_no_wait(self, resource_group):
        curr_dir = os.path.dirname(os.path.realpath(__file__))

        self.kwargs.update({
            'tf': os.path.join(curr_dir, 'simple_deploy.json').replace('\\', '\\\\'),
            'params': os.path.join(curr_dir, 'simple_deploy_parameters.json').replace('\\', '\\\\'),
            'dn': 'azure-cli-deployment'
        })

        self.cmd('group deployment create -g {rg} -n {dn} --template-file "{tf}" --parameters @"{params}" --no-wait',
                 checks=self.is_empty())

        self.cmd('group deployment wait -g {rg} -n {dn} --created',
                 checks=self.is_empty())

        self.cmd('group deployment show -g {rg} -n {dn}',
                 checks=self.check('properties.provisioningState', 'Succeeded'))


class DeploymentThruUriTest(ScenarioTest):

    @ResourceGroupPreparer(name_prefix='cli_test_deployment_uri')
    def test_group_deployment_thru_uri(self, resource_group):
        self.resource_group = resource_group
        curr_dir = os.path.dirname(os.path.realpath(__file__))
        # same copy of the sample template file under current folder, but it is uri based now
        self.kwargs.update({
            'tf': 'https://raw.githubusercontent.com/Azure/azure-cli/dev/src/azure-cli/azure/cli/command_modules/resource/tests/latest/simple_deploy.json',
            'params': os.path.join(curr_dir, 'simple_deploy_parameters.json').replace('\\', '\\\\')
        })
        self.kwargs['dn'] = self.cmd('group deployment create -g {rg} --template-uri "{tf}" --parameters @"{params}"', checks=[
            self.check('properties.provisioningState', 'Succeeded'),
            self.check('resourceGroup', '{rg}'),
        ]).get_output_in_json()['name']

        self.cmd('group deployment show -g {rg} -n {dn}',
                 checks=self.check('name', '{dn}'))

        self.cmd('group deployment delete -g {rg} -n {dn}')
        self.cmd('group deployment list -g {rg}',
                 checks=self.is_empty())


class ResourceMoveScenarioTest(ScenarioTest):
    @ResourceGroupPreparer(name_prefix='cli_test_resource_move_dest', parameter_name='resource_group_dest', key='rg2')
    @ResourceGroupPreparer(name_prefix='cli_test_resource_move_source', key='rg1')
    def test_resource_move(self, resource_group, resource_group_dest):
        self.kwargs.update({
            'nsg1': self.create_random_name('nsg-move', 20),
            'nsg2': self.create_random_name('nsg-move', 20)
        })

        self.kwargs['nsg1_id'] = self.cmd('network nsg create -n {nsg1} -g {rg1}').get_output_in_json()['NewNSG']['id']
        self.kwargs['nsg2_id'] = self.cmd('network nsg create -n {nsg2} -g {rg1}').get_output_in_json()['NewNSG']['id']

        self.cmd('resource move --ids {nsg1_id} {nsg2_id} --destination-group {rg2}')

        self.cmd('network nsg show -g {rg2} -n {nsg1}', checks=[
            self.check('name', '{nsg1}')])
        self.cmd('network nsg show -g {rg2} -n {nsg2}', checks=[
                 self.check('name', '{nsg2}')])


class PolicyScenarioTest(ScenarioTest):

    def cmdstring(self, basic, management_group=None, subscription=None):
        cmd = basic
        if (management_group):
            cmd = cmd + ' --management-group {mg}'
        if (subscription):
            cmd = cmd + ' --subscription {sub}'
        return cmd

    def applyPolicy(self):
        # create a policy assignment on a resource group
        self.kwargs.update({
            'pan': self.create_random_name('azurecli-test-policy-assignment', 40),
            'padn': self.create_random_name('test_assignment', 20)
        })

        self.cmd('policy assignment create --policy {pn} -n {pan} --display-name {padn} -g {rg} --params {params}', checks=[
            self.check('name', '{pan}'),
            self.check('displayName', '{padn}')
        ])

        # create a policy assignment using a built in policy definition name
        self.kwargs['pan2'] = self.create_random_name('azurecli-test-policy-assignment2', 40)
        self.kwargs['bip'] = '06a78e20-9358-41c9-923c-fb736d382a4d'

        self.cmd('policy assignment create --policy {bip} -n {pan2} --display-name {padn} -g {rg}', checks=[
            self.check('name', '{pan2}'),
            self.check('displayName', '{padn}')
        ])

        self.cmd('policy assignment delete -n {pan2} -g {rg}')

        # listing at subscription level won't find the assignment made at a resource group
        import jmespath
        try:
            self.cmd('policy assignment list', checks=self.check("length([?name=='{pan}'])", 0))
        except jmespath.exceptions.JMESPathTypeError:  # ok if query fails on None result
            pass

        # but enable --show-all works
        self.cmd('policy assignment list --disable-scope-strict-match', checks=self.check("length([?name=='{pan}'])", 1))

        # delete the assignment and validate it's gone
        self.cmd('policy assignment delete -n {pan} -g {rg}')
        self.cmd('policy assignment list --disable-scope-strict-match', checks=self.check("length([?name=='{pan}'])", 0))

    def applyPolicyAtScope(self, scope, policyId):
        # create a policy assignment at the given scope
        self.kwargs.update({
            'pol': policyId,
            'pan': self.create_random_name('cli-test-polassg', 24),   # limit is 24 characters at MG scope
            'padn': self.create_random_name('test_assignment', 20),
            'scope': scope
        })

        self.cmd('policy assignment create --policy {pol} -n {pan} --display-name {padn} --params {params} --scope {scope}', checks=[
            self.check('name', '{pan}'),
            self.check('displayName', '{padn}'),
            self.check('sku.name', 'A0'),
            self.check('sku.tier', 'Free')
        ])

        # delete the assignment and validate it's gone
        self.cmd('policy assignment delete -n {pan} --scope {scope}')
        self.cmd('policy assignment list --disable-scope-strict-match', checks=self.check("length([?name=='{pan}'])", 0))

    def resource_policy_operations(self, resource_group, management_group=None, subscription=None):
        curr_dir = os.path.dirname(os.path.realpath(__file__))

        self.kwargs.update({
            'pn': self.create_random_name('azure-cli-test-policy', 30),
            'pdn': self.create_random_name('test_policy', 20),
            'desc': 'desc_for_test_policy_123',
            'rf': os.path.join(curr_dir, 'sample_policy_rule.json').replace('\\', '\\\\'),
            'pdf': os.path.join(curr_dir, 'sample_policy_param_def.json').replace('\\', '\\\\'),
            'params': os.path.join(curr_dir, 'sample_policy_param.json').replace('\\', '\\\\'),
            'mode': 'Indexed'
        })
        if (management_group):
            self.kwargs.update({'mg': management_group})
        if (subscription):
            self.kwargs.update({'sub': subscription})

        # create a policy
        cmd = self.cmdstring('policy definition create -n {pn} --rules {rf} --params {pdf} --display-name {pdn} --description {desc} --mode {mode}', management_group, subscription)
        self.cmd(cmd, checks=[
            self.check('name', '{pn}'),
            self.check('displayName', '{pdn}'),
            self.check('description', '{desc}'),
            self.check('mode', '{mode}')
        ])

        # update it
        self.kwargs['desc'] = self.kwargs['desc'] + '_new'
        self.kwargs['pdn'] = self.kwargs['pdn'] + '_new'

        cmd = self.cmdstring('policy definition update -n {pn} --description {desc} --display-name {pdn}', management_group, subscription)
        self.cmd(cmd, checks=[
            self.check('description', '{desc}'),
            self.check('displayName', '{pdn}')
        ])

        # list and show it
        cmd = self.cmdstring('policy definition list', management_group, subscription)
        self.cmd(cmd, checks=self.check("length([?name=='{pn}'])", 1))

        cmd = self.cmdstring('policy definition show -n {pn}', management_group, subscription)
        self.cmd(cmd, checks=[
            self.check('name', '{pn}'),
            self.check('displayName', '{pdn}')
        ])

        # apply assignments
        if management_group:
            scope = '/providers/Microsoft.Management/managementGroups/{mg}'.format(mg=management_group)
            policy = '{scope}/providers/Microsoft.Authorization/policyDefinitions/{pn}'.format(pn=self.kwargs['pn'], scope=scope)
            self.applyPolicyAtScope(scope, policy)
        elif subscription:
            policy = '/subscriptions/{sub}/providers/Microsoft.Authorization/policyDefinitions/{pn}'.format(sub=subscription, pn=self.kwargs['pn'])
            self.applyPolicyAtScope('/subscriptions/{sub}'.format(sub=subscription), policy)
        else:
            self.applyPolicy()

        # delete the policy
        cmd = self.cmdstring('policy definition delete -n {pn}', management_group, subscription)
        self.cmd(cmd)
        time.sleep(10)  # ensure the policy is gone when run live.

        cmd = self.cmdstring('policy definition list', management_group, subscription)
        self.cmd(cmd, checks=self.check("length([?name=='{pn}'])", 0))

    def resource_policyset_operations(self, resource_group, management_group=None, subscription=None):
        curr_dir = os.path.dirname(os.path.realpath(__file__))

        self.kwargs.update({
            'pn': self.create_random_name('azure-cli-test-policy', 30),
            'pdn': self.create_random_name('test_policy', 20),
            'desc': 'desc_for_test_policy_123',
            'psn': self.create_random_name('azure-cli-test-policyset', 30),
            'psdn': self.create_random_name('test_policyset', 20),
            'ps_desc': 'desc_for_test_policyset_123',
            'rf': os.path.join(curr_dir, 'sample_policy_rule.json').replace('\\', '\\\\'),
            'psf': os.path.join(curr_dir, 'sample_policy_set.json').replace('\\', '\\\\'),
            'pdf': os.path.join(curr_dir, 'sample_policy_param_def.json').replace('\\', '\\\\')
        })
        if (management_group):
            self.kwargs.update({'mg': management_group})
        if (subscription):
            self.kwargs.update({'sub': subscription})

        # create a policy
        cmd = self.cmdstring('policy definition create -n {pn} --rules {rf} --params {pdf} --display-name {pdn} --description {desc}', management_group, subscription)
        policy = self.cmd(cmd).get_output_in_json()

        # create a policy set
        policyset = get_file_json(self.kwargs['psf'])
        policyset[0]['policyDefinitionId'] = policy['id']
        with open(os.path.join(curr_dir, 'sample_policy_set.json'), 'w') as outfile:
            json.dump(policyset, outfile)

        cmd = self.cmdstring('policy set-definition create -n {psn} --definitions @"{psf}" --display-name {psdn} --description {ps_desc}', management_group, subscription)
        self.cmd(cmd, checks=[
            self.check('name', '{psn}'),
            self.check('displayName', '{psdn}'),
            self.check('description', '{ps_desc}')
        ])

        # update it
        self.kwargs['ps_desc'] = self.kwargs['ps_desc'] + '_new'
        self.kwargs['psdn'] = self.kwargs['psdn'] + '_new'

        cmd = self.cmdstring('policy set-definition update -n {psn} --display-name {psdn} --description {ps_desc}', management_group, subscription)
        self.cmd(cmd, checks=[
            self.check('description', '{ps_desc}'),
            self.check('displayName', '{psdn}')
        ])

        # list and show it
        cmd = self.cmdstring('policy set-definition list', management_group, subscription)
        self.cmd(cmd, checks=self.check("length([?name=='{psn}'])", 1))

        cmd = self.cmdstring('policy set-definition show -n {psn}', management_group, subscription)
        self.cmd(cmd, checks=[
            self.check('name', '{psn}'),
            self.check('displayName', '{psdn}')
        ])

        # create a policy assignment on a resource group
        if not management_group and not subscription:
            self.kwargs.update({
                'pan': self.create_random_name('azurecli-test-policy-assignment', 40),
                'padn': self.create_random_name('test_assignment', 20)
            })

            self.cmd('policy assignment create -d {psn} -n {pan} --display-name {padn} -g {rg}', checks=[
                self.check('name', '{pan}'),
                self.check('displayName', '{padn}'),
                self.check('sku.name', 'A0'),
                self.check('sku.tier', 'Free'),
            ])

            # delete the assignment and validate it's gone
            self.cmd('policy assignment delete -n {pan} -g {rg}')
            self.cmd('policy assignment list --disable-scope-strict-match', checks=self.check("length([?name=='{pan}'])", 0))

        # delete the policy set
        cmd = self.cmdstring('policy set-definition delete -n {psn}', management_group, subscription)
        self.cmd(cmd)
        time.sleep(10)  # ensure the policy is gone when run live.

        cmd = self.cmdstring('policy set-definition list', management_group, subscription)
        self.cmd(cmd, checks=self.check("length([?name=='{psn}'])", 0))

        # delete the policy
        cmd = self.cmdstring('policy definition delete -n {pn}', management_group, subscription)
        self.cmd(cmd)
        time.sleep(10)  # ensure the policy is gone when run live.

        cmd = self.cmdstring('policy definition list', management_group, subscription)
        self.cmd(cmd, checks=self.check("length([?name=='{pn}'])", 0))

    @ResourceGroupPreparer(name_prefix='cli_test_policy')
    @AllowLargeResponse(8192)
    def test_resource_policy_default(self, resource_group):
        self.resource_policy_operations(resource_group)

    @record_only()
    @unittest.skip('mock doesnt work when the subscription comes from --scope')
    @ResourceGroupPreparer(name_prefix='cli_test_policy_subscription_id')
    @AllowLargeResponse()
    def test_resource_policy_subscription_id(self, resource_group):
        # under playback, we mock it so the subscription id will be '00000000...' and it will match
        # the same sanitized value in the recording
        if not self.in_recording:
            with mock.patch('azure.cli.command_modules.resource.custom._get_subscription_id_from_subscription',
                            return_value=MOCKED_SUBSCRIPTION_ID):
                self.resource_policy_operations(resource_group, None, 'e8a0d3c2-c26a-4363-ba6b-f56ac74c5ae0')
        else:
            self.resource_policy_operations(resource_group, None, 'e8a0d3c2-c26a-4363-ba6b-f56ac74c5ae0')

    @AllowLargeResponse(8192)
    def test_show_built_in_policy(self):
        # get the list of builtins, then retrieve each via show and validate the results match
        results = self.cmd('policy definition list --query "[?policyType==\'BuiltIn\']"').get_output_in_json()
        for i, result in enumerate(results):
            self.kwargs['pn'] = result['name']
            self.kwargs['dn'] = result['displayName']
            self.kwargs['desc'] = result['description']
            self.kwargs['id'] = result['id']
            self.cmd('policy definition show -n {pn}', checks=[
                self.check('name', '{pn}'),
                self.check('description', '{desc}'),
                self.check('displayName', '{dn}'),
                self.check('id', '{id}')
            ])


class ManagedAppDefinitionScenarioTest(ScenarioTest):
    @ResourceGroupPreparer()
    def test_managedappdef_inline(self, resource_group):
        curr_dir = os.path.dirname(os.path.realpath(__file__))

        self.kwargs.update({
            'loc': 'eastus',
            'adn': self.create_random_name('testappdefname', 20),
            'addn': self.create_random_name('test_appdef', 20),
            'ad_desc': 'test_appdef_123',
            'auth': '5e91139a-c94b-462e-a6ff-1ee95e8aac07:8e3af657-a8ff-443c-a75c-2fe8c4bcb635',
            'lock': 'None',
            'ui_file': os.path.join(curr_dir, 'sample_create_ui_definition.json').replace('\\', '\\\\'),
            'main_file': os.path.join(curr_dir, 'sample_main_template.json').replace('\\', '\\\\')
        })

        # create a managedapp definition with inline params for create-ui-definition and main-template
        self.kwargs['ad_id'] = self.cmd('managedapp definition create -n {adn} --create-ui-definition @"{ui_file}" --main-template @"{main_file}" --display-name {addn} --description {ad_desc} -l {loc} -a {auth} --lock-level {lock} -g {rg}', checks=[
            self.check('name', '{adn}'),
            self.check('displayName', '{addn}'),
            self.check('description', '{ad_desc}'),
            self.check('authorizations[0].principalId', '5e91139a-c94b-462e-a6ff-1ee95e8aac07'),
            self.check('authorizations[0].roleDefinitionId', '8e3af657-a8ff-443c-a75c-2fe8c4bcb635'),
            self.check('artifacts[0].name', 'ApplicationResourceTemplate'),
            self.check('artifacts[0].type', 'Template'),
            self.check('artifacts[1].name', 'CreateUiDefinition'),
            self.check('artifacts[1].type', 'Custom')
        ]).get_output_in_json()['id']

        self.cmd('managedapp definition list -g {rg}',
                 checks=self.check('[0].name', '{adn}'))

        self.cmd('managedapp definition show --ids {ad_id}', checks=[
            self.check('name', '{adn}'),
            self.check('displayName', '{addn}'),
            self.check('description', '{ad_desc}'),
            self.check('authorizations[0].principalId', '5e91139a-c94b-462e-a6ff-1ee95e8aac07'),
            self.check('authorizations[0].roleDefinitionId', '8e3af657-a8ff-443c-a75c-2fe8c4bcb635'),
            self.check('artifacts[0].name', 'ApplicationResourceTemplate'),
            self.check('artifacts[0].type', 'Template'),
            self.check('artifacts[1].name', 'CreateUiDefinition'),
            self.check('artifacts[1].type', 'Custom')
        ])

        self.cmd('managedapp definition delete -g {rg} -n {adn}')
        self.cmd('managedapp definition list -g {rg}', checks=self.is_empty())


# TODO: Change back to ScenarioTest and re-record when issue #5110 is fixed.
class ManagedAppScenarioTest(LiveScenarioTest):
    @ResourceGroupPreparer()
    def test_managedapp(self, resource_group):

        self.kwargs.update({
            'loc': 'westcentralus',
            'adn': 'testappdefname',
            'addn': 'test_appdef_123',
            'ad_desc': 'test_appdef_123',
            'uri': 'https://wud.blob.core.windows.net/appliance/SingleStorageAccount.zip',
            'auth': '5e91139a-c94b-462e-a6ff-1ee95e8aac07:8e3af657-a8ff-443c-a75c-2fe8c4bcb635',
            'lock': 'None',
            'sub': self.get_subscription_id()
        })

        self.kwargs['ad_id'] = self.cmd('managedapp definition create -n {adn} --package-file-uri {uri} --display-name {addn} --description {ad_desc} -l {loc} -a {auth} --lock-level {lock} -g {rg}').get_output_in_json()['id']

        # create a managedapp
        self.kwargs.update({
            'man': 'mymanagedapp',
            'ma_loc': 'westcentralus',
            'ma_kind': 'servicecatalog',
            'ma_rg': self.create_random_name('climanagedapp', 25)
        })
        self.kwargs['ma_rg_id'] = '/subscriptions/{sub}/resourceGroups/{ma_rg}'.format(**self.kwargs)

        self.kwargs['ma_id'] = self.cmd('managedapp create -n {man} -g {rg} -l {ma_loc} --kind {ma_kind} -m {ma_rg_id} -d {ad_id}', checks=[
            self.check('name', '{man}'),
            self.check('type', 'Microsoft.Solutions/applications'),
            self.check('kind', 'servicecatalog'),
            self.check('managedResourceGroupId', '{ma_rg_id}')
        ]).get_output_in_json()['id']

        self.cmd('managedapp list -g {rg}', checks=self.check('[0].name', '{man}'))

        self.cmd('managedapp show --ids {ma_id}', checks=[
            self.check('name', '{man}'),
            self.check('type', 'Microsoft.Solutions/applications'),
            self.check('kind', 'servicecatalog'),
            self.check('managedResourceGroupId', '{ma_rg_id}')
        ])

        self.cmd('managedapp delete -g {rg} -n {man}')
        self.cmd('managedapp list -g {rg}', checks=self.is_empty())


class CrossRGDeploymentScenarioTest(ScenarioTest):

    @ResourceGroupPreparer(name_prefix='cli_test_cross_rg_alt', parameter_name='resource_group_cross')
    @ResourceGroupPreparer(name_prefix='cli_test_cross_rg_deploy')
    def test_group_deployment_crossrg(self, resource_group, resource_group_cross):
        curr_dir = os.path.dirname(os.path.realpath(__file__))

        self.kwargs.update({
            'rg1': resource_group,
            'rg2': resource_group_cross,
            'tf': os.path.join(curr_dir, 'crossrg_deploy.json').replace('\\', '\\\\'),
            'dn': self.create_random_name('azure-cli-crossrgdeployment', 40),
            'sa1': create_random_name(prefix='crossrg'),
            'sa2': create_random_name(prefix='crossrg')
        })

        self.cmd('group deployment validate -g {rg1} --template-file "{tf}" --parameters CrossRg={rg2} StorageAccountName1={sa1} StorageAccountName2={sa2}', checks=[
            self.check('properties.provisioningState', 'Succeeded')
        ])
        self.cmd('group deployment create -g {rg1} -n {dn} --template-file "{tf}" --parameters CrossRg={rg2}', checks=[
            self.check('properties.provisioningState', 'Succeeded'),
            self.check('resourceGroup', '{rg1}'),
        ])
        self.cmd('group deployment list -g {rg1}', checks=[
            self.check('[0].name', '{dn}'),
            self.check('[0].resourceGroup', '{rg1}')
        ])
        self.cmd('group deployment show -g {rg1} -n {dn}', checks=[
            self.check('name', '{dn}'),
            self.check('resourceGroup', '{rg1}')
        ])
        self.cmd('group deployment operation list -g {rg1} -n {dn}', checks=[
            self.check('length([])', 3),
            self.check('[0].resourceGroup', '{rg1}')
        ])


class InvokeActionTest(ScenarioTest):
    @ResourceGroupPreparer(name_prefix='cli_test_invoke_action')
    def test_invoke_action(self, resource_group):

        self.kwargs.update({
            'vm': self.create_random_name('cli-test-vm', 30),
            'user': 'ubuntu',
            'pass': self.create_random_name('Longpassword#1', 30)
        })

        self.kwargs['vm_id'] = self.cmd('vm create -g {rg} -n {vm} --use-unmanaged-disk --image UbuntuLTS --admin-username {user} --admin-password {pass} --authentication-type password --nsg-rule None').get_output_in_json()['id']

        self.cmd('resource invoke-action --action powerOff --ids {vm_id}')
        self.cmd('resource invoke-action --action generalize --ids {vm_id}')
        self.cmd('resource invoke-action --action deallocate --ids {vm_id}')

        self.kwargs['request_body'] = '{\\"vhdPrefix\\":\\"myPrefix\\",\\"destinationContainerName\\":\\"container\\",\\"overwriteVhds\\":\\"true\\"}'

        self.cmd('resource invoke-action --action capture --ids {vm_id} --request-body {request_body}')


class GlobalIdsScenarioTest(ScenarioTest):
    @ResourceGroupPreparer(name_prefix='cli_test_global_ids')
    def test_global_ids(self, resource_group):

        self.kwargs.update({
            'vnet': 'vnet1'
        })
        self.kwargs['vnet_id'] = self.cmd('network vnet create -g {rg} -n {vnet}').get_output_in_json()['newVNet']['id']
        # command will fail if the other parameters were actually used
        self.cmd('network vnet show --subscription fakesub --resource-group fakerg -n fakevnet --ids {vnet_id}')


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer


class CognitiveServicesApiPropertiesTests(ScenarioTest):
    @ResourceGroupPreparer()
    def test_cognitiveservices_create_api_properties(self, resource_group):
        sname = self.create_random_name(prefix='cs_cli_test_', length=16)

        self.kwargs.update({
            'sname': sname,
            'kind': 'QnAMaker',
            'sku': 'S0',
            'location': 'westus',
            'apiProperties': 'qnaRuntimeEndpoint=https://cs-cli-test-qnamaker.azurewebsites.net',
            'apiPropertiesJson': '{\\\"qnaRuntimeEndpoint\\\":\\\"https://cs-cli-test-qnamaker.azurewebsites.net\\\"}',
        })

        # test to create cognitive services account
        self.cmd('az cognitiveservices account create -n {sname} -g {rg} --kind {kind} --sku {sku} -l {location} '
                 '--api-properties {apiProperties} --yes',
                 checks=[self.check('name', '{sname}'),
                         self.check('location', '{location}'),
                         self.check('sku.name', '{sku}'),
                         self.check('properties.provisioningState', 'Succeeded')])

        # delete the cognitive services account
        ret = self.cmd('az cognitiveservices account delete -n {sname} -g {rg}')
        self.assertEqual(ret.exit_code, 0)

        # test to create cognitive services account
        self.cmd('az cognitiveservices account create -n {sname} -g {rg} --kind {kind} --sku {sku} -l {location} '
                 '--api-properties {apiPropertiesJson} --yes',
                 checks=[self.check('name', '{sname}'),
                         self.check('location', '{location}'),
                         self.check('sku.name', '{sku}'),
                         self.check('properties.provisioningState', 'Succeeded')])

        # delete the cognitive services account
        ret = self.cmd('az cognitiveservices account delete -n {sname} -g {rg}')
        self.assertEqual(ret.exit_code, 0)


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import sys

import azure.cli.core.telemetry as telemetry
from knack.util import CLIError
from knack.log import get_logger

logger = get_logger(__name__)
# pylint: disable=unnecessary-pass


# Error types in AzureCLI are from different sources, and there are many general error types like CLIError, AzureError.
# Besides, many error types with different names are actually showing the same kind of error.
# For example, CloudError, CLIError and ValidationError all could be a resource-not-found error.
# Therefore, here we define the new error classes to map and categorize all of the error types from different sources.


# region: Base Layer
# Base class for all the AzureCLI defined error classes.
class AzCLIError(CLIError):
    """ Base class for all the AzureCLI defined error classes.
    DO NOT raise this error class in your codes. """

    def __init__(self, error_msg, recommendation=None):
        # error message
        self.error_msg = error_msg

        # set recommendations to fix the error if the message is not actionable,
        # they will be printed to users after the error message, one recommendation per line
        self.recommendations = []
        self.set_recommendation(recommendation)

        # exception trace for the error
        self.exception_trace = None
        super().__init__(error_msg)

    def set_recommendation(self, recommendation):
        if isinstance(recommendation, str):
            self.recommendations.append(recommendation)
        elif isinstance(recommendation, list):
            self.recommendations.extend(recommendation)

    def set_exception_trace(self, exception_trace):
        self.exception_trace = exception_trace

    def print_error(self):
        from azure.cli.core.azlogging import CommandLoggerContext
        with CommandLoggerContext(logger):
            # print error type and error message
            message = '{}: {}'.format(self.__class__.__name__, self.error_msg)
            logger.error(message)
            # print exception trace if there is
            if self.exception_trace:
                logger.exception(self.exception_trace)
            # print recommendations to action
            if self.recommendations:
                for recommendation in self.recommendations:
                    print(recommendation, file=sys.stderr)

    def send_telemetry(self):
        telemetry.set_error_type(self.__class__.__name__)
# endregion


# region: Second Layer
# Main categories of the AzureCLI error types, used for Telemetry analysis
class UserFault(AzCLIError):
    """ Users should be responsible for the errors.
    DO NOT raise this error class in your codes. """
    def send_telemetry(self):
        super().send_telemetry()
        telemetry.set_user_fault(self.error_msg)


class ServiceError(AzCLIError):
    """ Azure Services should be responsible for the errors.
    DO NOT raise this error class in your codes. """
    def send_telemetry(self):
        super().send_telemetry()
        telemetry.set_failure(self.error_msg)


class ClientError(AzCLIError):
    """ AzureCLI should be responsible for the errors.
    DO NOT raise this error class in your codes. """
    def send_telemetry(self):
        super().send_telemetry()
        telemetry.set_failure(self.error_msg)
        if self.exception_trace:
            telemetry.set_exception(self.exception_trace, '')


class UnknownError(AzCLIError):
    """ Unclear errors, could not know who should be responsible for the errors.
    DO NOT raise this error class in your codes. """
    def send_telemetry(self):
        super().send_telemetry()
        telemetry.set_failure(self.error_msg)

    def print_error(self):
        from azure.cli.core.azlogging import CommandLoggerContext
        with CommandLoggerContext(logger):
            # print only error message (no error type)
            logger.error(self.error_msg)
            # print recommendations to action
            if self.recommendations:
                for recommendation in self.recommendations:
                    print(recommendation, file=sys.stderr)
# endregion


# region: Third Layer
# Specific categories of the AzureCLI error types
# Raise the error classes here in your codes. Avoid using fallback error classes unless you can not find a proper one.
# Command related error types
class CommandNotFoundError(UserFault):
    """ Command is misspelled or not recognized by AzureCLI. """
    pass


# Argument related error types
class UnrecognizedArgumentError(UserFault):
    """ Argument is misspelled or not recognized by AzureCLI. """
    pass


class RequiredArgumentMissingError(UserFault):
    """ Required argument is not specified. """
    pass


class MutuallyExclusiveArgumentError(UserFault):
    """ Arguments can not be specified together. """
    pass


class InvalidArgumentValueError(UserFault):
    """ Argument value is not valid. """
    pass


class ArgumentUsageError(UserFault):
    """ Fallback of the argument usage related errors.
    Avoid using this class unless the error can not be classified
    into the Argument related specific error types. """
    pass


# Response related error types
class BadRequestError(UserFault):
    """ Bad request from client: 400 error """
    pass


class UnauthorizedError(UserFault):
    """ Unauthorized request: 401 error """
    pass


class ForbiddenError(UserFault):
    """ Service refuse to response: 403 error """
    pass


class ResourceNotFoundError(UserFault):
    """ Can not find Azure resources: 404 error """
    pass


class AzureInternalError(ServiceError):
    """ Azure service internal error: 5xx error """
    pass


class AzureResponseError(UserFault):
    """ Fallback of the response related errors.
    Avoid using this class unless the error can not be classified
    into the Response related specific error types. """
    pass


# Request related error types
class AzureConnectionError(UserFault):
    """ Connection issues like connection timeout, aborted or broken. """
    pass


class ClientRequestError(UserFault):
    """ Fallback of the request related errors. Error occurs while attempting
    to make a request to the service. No request is sent.
    Avoid using this class unless the error can not be classified
    into the Request related specific errors types. """
    pass


# File operation related error types
class FileOperationError(UserFault):
    """ For file or directory operation related errors. """
    pass


# Keyboard interrupt error type
class ManualInterrupt(UserFault):
    """ Keyboard interrupt. """
    pass


# ARM template related error types
class InvalidTemplateError(UserFault):
    """ ARM template validation fails. It could be caused by incorrect template files or parameters """
    pass


class DeploymentError(UserFault):
    """ ARM template deployment fails. Template file is valid, and error occurs in deployment. """
    pass


# Validation related error types
class ValidationError(UserFault):
    """ Fallback of the errors in validation functions.
    Avoid using this class unless the error can not be classified into
    the Argument, Request and Response related specific error types. """
    pass


class UnclassifiedUserFault(UserFault):
    """ Fallback of the UserFault related error types.
    Avoid using this class unless the error can not be classified into
    the UserFault related specific error types.
    """
    def print_error(self):
        from azure.cli.core.azlogging import CommandLoggerContext
        with CommandLoggerContext(logger):
            # print only error message (no error type)
            logger.error(self.error_msg)
            # print recommendations to action
            if self.recommendations:
                for recommendation in self.recommendations:
                    print(recommendation, file=sys.stderr)


# CLI internal error type
class CLIInternalError(ClientError):
    """ AzureCLI internal error """
    pass

# endregion



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import time
import unittest
from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer


class AzureSignalRServiceScenarioTest(ScenarioTest):
    @ResourceGroupPreparer(random_name_length=20)
    def test_signalr_commands(self, resource_group):
        signalr_name = self.create_random_name('signalr', 16)
        sku = 'Standard_S1'
        unit_count = 1
        location = 'eastus'
        tags_key = 'key'
        tags_val = 'value'
        service_mode = 'Classic'
        enable_message_logs_input = 'true'
        enable_message_logs_check = True
        allowed_origins = ['http://example1.com', 'http://example2.com']
        updated_sku = 'Free_F1'
        updated_tags_val = 'value2'
        update_service_mode = 'Serverless'
        update_enable_message_logs_input = 'false'
        update_enable_message_logs_check = False
        added_allowed_origins = ['http://example3.com', 'http://example4.com']

        self.kwargs.update({
            'location': location,
            'signalr_name': signalr_name,
            'sku': sku,
            'unit_count': unit_count,
            'tags': '{}={}'.format(tags_key, tags_val),
            'enable_message_logs_input': enable_message_logs_input,
            'enable_message_logs_check': enable_message_logs_check,
            'service_mode': service_mode,
            'allowed_origins': ' '.join(allowed_origins),
            'updated_sku': updated_sku,
            'updated_tags': '{}={}'.format(tags_key, updated_tags_val),
            'update_service_mode': update_service_mode,
            'update_enable_message_logs_input': update_enable_message_logs_input,
            'update_enable_message_logs_check': update_enable_message_logs_check,
            'added_allowed_origins': ' '.join(added_allowed_origins),
            'default_action': "Deny",
        })

        # Test create
        self.cmd('az signalr create -n {signalr_name} -g {rg} --sku {sku} --unit-count {unit_count} -l {location} --tags {tags} --service-mode {service_mode} --enable-message-logs {enable_message_logs_input} --allowed-origins {allowed_origins} --default-action {default_action}',
                 checks=[
                     self.check('name', '{signalr_name}'),
                     self.check('location', '{location}'),
                     self.check('provisioningState', 'Succeeded'),
                     self.check('sku.name', '{sku}'),
                     self.check('sku.capacity', '{unit_count}'),
                     self.check('tags.{}'.format(tags_key), tags_val),
                     self.check('features[0].value', '{service_mode}'),
                     self.check('features[2].value', '{enable_message_logs_check}'),
                     self.check('cors.allowedOrigins', allowed_origins),
                     self.exists('hostName'),
                     self.exists('publicPort'),
                     self.exists('serverPort'),
                     self.check('networkAcLs.defaultAction', '{default_action}')
                 ])

        # Test show
        self.cmd('az signalr show -n {signalr_name} -g {rg}', checks=[
            self.check('name', '{signalr_name}'),
            self.check('location', '{location}'),
            self.check('provisioningState', 'Succeeded'),
            self.check('sku.name', '{sku}'),
            self.check('sku.capacity', '{unit_count}'),
            self.check('features[0].value', '{service_mode}'),
            self.check('features[2].value', '{enable_message_logs_check}'),
            self.check('cors.allowedOrigins', allowed_origins),
            self.exists('hostName'),
            self.exists('publicPort'),
            self.exists('serverPort'),
            self.exists('externalIp'),
            self.check('networkAcLs.defaultAction', '{default_action}')
        ])

        # Test list
        self.cmd('az signalr list -g {rg}', checks=[
            self.check('[0].name', '{signalr_name}'),
            self.check('[0].location', '{location}'),
            self.check('[0].provisioningState', 'Succeeded'),
            self.check('[0].sku.name', '{sku}'),
            self.check('[0].sku.capacity', '{unit_count}'),
            self.check('[0].features[0].value', '{service_mode}'),
            self.check('[0].features[2].value', '{enable_message_logs_check}'),
            self.check('[0].cors.allowedOrigins', allowed_origins),
            self.exists('[0].hostName'),
            self.exists('[0].publicPort'),
            self.exists('[0].serverPort'),
            self.exists('[0].externalIp'),
            self.check('[0].networkAcLs.defaultAction', '{default_action}')
        ])

        # Test update
        self.cmd('az signalr update -n {signalr_name} -g {rg} --sku {updated_sku} --tags {updated_tags} --service-mode {update_service_mode} --enable-message-logs {update_enable_message_logs_input}',
                 checks=[
                     self.check('name', '{signalr_name}'),
                     self.check('location', '{location}'),
                     self.check('provisioningState', 'Succeeded'),
                     self.check('sku.name', '{updated_sku}'),
                     self.check('tags.{}'.format(tags_key), updated_tags_val),
                     self.check('features[0].value', '{update_service_mode}'),
                     self.check('features[2].value', '{update_enable_message_logs_check}'),
                     self.check('cors.allowedOrigins', allowed_origins),
                     self.exists('hostName'),
                     self.exists('publicPort'),
                     self.exists('serverPort')
                 ])

        # Test CORS operations
        self.cmd('az signalr cors remove -n {signalr_name} -g {rg} --allowed-origins {allowed_origins}', checks=[
            self.check('cors.allowedOrigins[0]', '*')
        ])

        self.cmd('az signalr cors add -n {signalr_name} -g {rg} --allowed-origins {added_allowed_origins}', checks=[
            self.check('cors.allowedOrigins', ['*'] + added_allowed_origins)
        ])

        # Test key list
        self.cmd('az signalr key list -n {signalr_name} -g {rg}', checks=[
            self.exists('primaryKey'),
            self.exists('secondaryKey')
        ])

        # Test key renew
        self.cmd('az signalr key renew -n {signalr_name} -g {rg} --key-type secondary', checks=[
            self.exists('primaryKey'),
            self.exists('secondaryKey')
        ])
